@@ -69,20 +69,20 @@ static tmsgport DM_MsgPort;
 static tmenu DM_Menu;
 static tscreen DM_Screen;
 static twindow DM_Window, DM_user, DM_pass;
-static TW_CONST char *DM_Display;
+static const char *DM_Display;
 static byte DM_Kill, quiet, oneshot, logged_in;
 
 static TW_VOLATILE pid_t ServerPid = (pid_t)-1, AttachPid = (pid_t)-1;
 
-static char *TW_CONST *Args;
-static TW_CONST char *hw_name = "--hw=tty", *title, *TwEnvRC = NULL;
+static char *const *Args;
+static const char *hw_name = "--hw=tty", *title, *TwEnvRC = NULL;
 static byte use_twdisplay = DM_AUTO, Privileges = TW_PRIV_NONE;
 
 typedef struct s_data {
   char txt[30];
   byte len;
   byte x;
-} * data;
+} *data;
 
 static struct s_data user, pass;
 
@@ -97,7 +97,7 @@ static void Usage(void) {
                   " --attach                use \"twattach\" to start display\n"
                   " --display               use \"twdisplay\" to start display\n"
                   "                             (default unless --hw=tty)\n"
-                  " --envrc                 tell twin to run .twenvrc.sh to get environment\n"
+                  " --envrc                 tell twin to run twenvrc.sh to get environment\n"
                   " --suidroot              tell twin to keep suid root privileges\n"
                   " --sgidtty               tell twin to keep sgid tty privileges\n"
                   " --title=<title>         set window title\n"
@@ -109,7 +109,7 @@ static void ShowVersion(void) {
 }
 
 static void ParseArgs(void) {
-  TW_CONST char *s;
+  const char *s;
   while ((s = *Args)) {
     if (!strcmp(s, "-h") || !Tw_option_strcmp(s, "-help")) {
       Usage();
@@ -238,7 +238,7 @@ static void shortsleep(void) {
 }
 
 static byte InitAttach(void) {
-  char *attach, *path_attach;
+  const char *attach, *path_attach;
   char buff[] = "--twin@:\0\0\0";
 
   if (use_twdisplay == DM_ATTACH ||
@@ -294,17 +294,18 @@ static byte InitClient(void) {
 
   if (!title) {
     struct utsname buf;
-    if (uname(&buf) >= 0 && (tbuf = (char *)TwAllocMem(12 + strlen(buf.nodename)))) {
-      sprintf(tbuf, "Welcome at %s", buf.nodename);
+    size_t tbuf_len;
+    if (uname(&buf) >= 0 && (tbuf = (char *)TwAllocMem(tbuf_len = 12 + strlen(buf.nodename)))) {
+      snprintf(tbuf, tbuf_len, "Welcome at %s", buf.nodename);
       title = tbuf;
     } else
       title = "Login Manager";
   }
 
   if ((DM_MsgPort = TwCreateMsgPort(2, "DM")) &&
-      (DM_Menu = TwCreateMenu(TCOL(tblack, twhite), TCOL(tblack, tgreen),
-                              TCOL(thigh | tblack, twhite), TCOL(thigh | tblack, tblack),
-                              TCOL(tred, twhite), TCOL(tred, tgreen), (byte)0)) &&
+      (DM_Menu =
+           TwCreateMenu(TCOL(tblack, twhite), TCOL(tblack, tgreen), TCOL(tBLACK, twhite),
+                        TCOL(tBLACK, tblack), TCOL(tred, twhite), TCOL(tred, tgreen), (byte)0)) &&
       TwItem4MenuCommon(DM_Menu) &&
       (TwInfo4Menu(DM_Menu, TW_ROW_ACTIVE, 61,
                    " Twin Display Manager. Enter user name and password to login.", NULL),
@@ -321,17 +322,16 @@ static byte InitClient(void) {
            TW_WINDOWFL_CURSOR_ON | TW_WINDOWFL_USEROWS | TW_WINDOWFL_ROWS_DEFCOL, 30, 1, 0)) &&
 
       TwCreateButtonGadget(DM_Window, 7, 1, " Login ", TW_GADGETFL_USETEXT, DM_GADGET_LOGIN,
-                           TCOL(twhite, tblue), TCOL(thigh | twhite, tgreen),
-                           TCOL(thigh | tblack, tgreen), 5, 12) &&
+                           TCOL(twhite, tblue), TCOL(tWHITE, tgreen), TCOL(tBLACK, tgreen), 5,
+                           12) &&
       TwCreateButtonGadget(DM_Window, 7, 1, " Clear ", TW_GADGETFL_USETEXT, DM_GADGET_CLEAR,
-                           TCOL(twhite, tblue), TCOL(thigh | twhite, tgreen),
-                           TCOL(thigh | tblack, tgreen), 16, 12) &&
+                           TCOL(twhite, tblue), TCOL(tWHITE, tgreen), TCOL(tBLACK, tgreen), 16,
+                           12) &&
       TwCreateButtonGadget(DM_Window, 7, 1, " Leave ", TW_GADGETFL_USETEXT, DM_GADGET_CONSOLE,
-                           TCOL(twhite, tblue), TCOL(thigh | twhite, tgreen),
-                           TCOL(thigh | tblack, tgreen), 27, 12)
+                           TCOL(twhite, tblue), TCOL(tWHITE, tgreen), TCOL(tBLACK, tgreen), 27, 12)
 
   ) {
-    TwSetColTextWindow(DM_Window, TCOL(thigh | tred, tblue));
+    TwSetColTextWindow(DM_Window, TCOL(tRED, tblue));
     TwGotoXYWindow(DM_Window, 7, 8);
     TwWriteCharsetWindow(DM_Window, 26, "L O G I N   F A I L E D  !");
 
@@ -567,16 +567,16 @@ static void HandleGadget(tevent_gadget E) {
   }
 }
 
-static TW_RETSIGTYPE SignalPanic(int n) {
+static void SignalPanic(int n) {
   signal(n, SIG_DFL);
   quit();
-  TW_RETFROMSIGNAL(0);
 }
 
-static TW_RETSIGTYPE SignalChild(int n) {
+static void SignalChild(int n) {
   pid_t pid;
   int status;
 
+  (void)n;
   signal(SIGCHLD, SignalChild);
 
   while ((pid = Tw_wait3(&status, WNOHANG, (struct rusage *)0)) != 0 && pid != (pid_t)-1) {
@@ -585,7 +585,6 @@ static TW_RETSIGTYPE SignalChild(int n) {
         AttachPid = (pid_t)-1;
     }
   }
-  TW_RETFROMSIGNAL(0);
 }
 
 static void InitSignals(void) {
@@ -614,9 +613,10 @@ static void InitSignals(void) {
 int main(int argc, char *argv[]) {
   fd_set fset;
   int fd;
-  tmsg Msg;
+  tmsg msg;
   uldat err;
 
+  (void)argc;
   Args = argv + 1;
 
   ParseArgs();
@@ -643,24 +643,24 @@ int main(int argc, char *argv[]) {
     while (AttachPid != (pid_t)-1 && ServerPid != (pid_t)-1 && logged_in == tfalse &&
            !TwInPanic()) {
 
-      while ((Msg = TwReadMsg(tfalse)))
-        switch (Msg->Type) {
+      while ((msg = TwReadMsg(tfalse)))
+        switch (msg->Type) {
         case TW_MSG_WIDGET_KEY:
-          HandleKey(&Msg->Event.EventKeyboard);
+          HandleKey(&msg->Event.EventKeyboard);
           break;
         case TW_MSG_WIDGET_GADGET:
-          HandleGadget(&Msg->Event.EventGadget);
+          HandleGadget(&msg->Event.EventGadget);
           break;
         case TW_MSG_SELECTION:
           /*
-           * send Msg->Event.EventSelection.W as ReqPrivate field,
+           * send msg->Event.EventSelection.W as ReqPrivate field,
            * so that we will get it back in TW_MSG_SELECTIONNOTIFY message
            * without having to store it manually
            */
-          TwRequestSelection(TwGetOwnerSelection(), Msg->Event.EventSelection.W);
+          TwRequestSelection(TwGetOwnerSelection(), msg->Event.EventSelection.W);
           break;
         case TW_MSG_SELECTIONNOTIFY: {
-          tevent_selectionnotify E = &Msg->Event.EventSelectionNotify;
+          tevent_selectionnotify E = &msg->Event.EventSelectionNotify;
 
           /* react as for keypresses */
           if (E->Magic == TW_SEL_UTF8MAGIC) {
@@ -684,8 +684,8 @@ int main(int argc, char *argv[]) {
     /* wait for user to logout */
     while (AttachPid != (pid_t)-1 && ServerPid != (pid_t)-1 && logged_in == ttrue && !TwInPanic()) {
 
-      while ((Msg = TwReadMsg(tfalse)))
-        ;
+      while ((msg = TwReadMsg(tfalse))) {
+      }
 
       /* sleep until a message or a signal arrives (as above) */
       FD_SET(fd, &fset);
@@ -697,10 +697,10 @@ int main(int argc, char *argv[]) {
     }
   }
 
-  if ((err = TwErrno))
+  if ((err = TwErrno) != 0) {
     fprintf(stderr, "%s: libtw error: %s%s\n", argv[0], TwStrError(err),
             TwStrErrorDetail(err, TwErrnoDetail));
-
+  }
   quit();
 
   /*NOTREACHED*/
