@@ -9,7 +9,7 @@
  *
  */
 
-trune Tutf_CP437_to_UTF_32[0x100] = {
+const trune Tutf_CP437_to_UTF_32[0x100] = {
 #define EL(x) T_UTF(UTF_32, x),
     T_LIST(CP437, EL)
 #undef EL
