@@ -16,13 +16,13 @@
  * display.c and the whole twin sources, in particular hw_multi.c
  */
 
-char *CloneStr(const char *s);
-char *CloneStrL(const char *s, uldat len);
+char *CloneStr(const char *s) NOTHROW;
+char *CloneStrL(const char *s, uldat len) NOTHROW;
 
 byte AllHWCanDragAreaNow(dat Left, dat Up, dat Rgt, dat Dwn, dat DstLeft, dat DstUp);
 void DragAreaHW(dat Xstart, dat Ystart, dat Xend, dat Yend, dat DstXstart, dat DstYstart);
 
-void ResizeDisplayPrefer(display_hw);
+void ResizeDisplayPrefer(Tdisplay);
 
 void PanicHW(void);
 void FlushHW(void);
@@ -31,25 +31,24 @@ void ConfigureHW(udat resource, byte todefault, udat value);
 void SetPaletteHW(udat N, udat R, udat G, udat B);
 void ResetPaletteHW(void);
 
-obj TwinSelectionGetOwner(void);
-void TwinSelectionSetOwner(obj Owner, tany Time, tany Frac);
+Tobj TwinSelectionGetOwner(void);
+void TwinSelectionSetOwner(Tobj Owner, tany Time, tany Frac);
 #define SEL_CURRENTTIME ((tany)0)
-void TwinSelectionNotify(obj Requestor, uldat ReqPrivate, uldat Magic, const char MIME[MAX_MIMELEN],
-                         Chars Data);
-void TwinSelectionRequest(obj Requestor, uldat ReqPrivate, obj Owner);
+void TwinSelectionNotify(Tobj Requestor, uldat ReqPrivate, e_id Magic, Chars mime, Chars data);
+void TwinSelectionRequest(Tobj Requestor, uldat ReqPrivate, Tobj Owner);
 
 void SelectionExport(void);
 void SelectionImport(void);
-byte SelectionStore(uldat magic, const char mime[MAX_MIMELEN], Chars data);
+bool SelectionStore(e_id magic, Chars mime, Chars data);
 byte SelectionAppend(Chars data);
 byte SelectionAppendRune(trune rune);
 byte SelectionAppendRunes(TRunes runes);
 
-byte KeyboardEventCommon(udat Code, udat ShiftFlags, udat Len, const char *Seq);
-byte MouseEventCommon(dat x, dat y, dat dx, dat dy, udat IdButtons);
+bool KeyboardEventCommon(Tdisplay hw, udat Code, udat ShiftFlags, udat Len, const char *Seq);
+byte MouseEventCommon(Tdisplay hw, dat x, dat y, dat dx, dat dy, udat IdButtons);
 
-dat GetDisplayWidth(void);
-dat GetDisplayHeight(void);
+dat GetDisplayWidth(void) NOTHROW;
+dat GetDisplayHeight(void) NOTHROW;
 
 byte InitTransUser(void);
 
@@ -57,13 +56,12 @@ byte InitTransUser(void);
  * these comes either from display.c or from main.c/util.c
  */
 void Quit(int status);
-void GainPrivileges(void);
+void gainPrivileges(void);
 
 #ifndef SS
 #define SS "%." STR(TW_SMALLBUFF) "s"
 #endif
 
-int printk(const char *format, ...);
 int flushk(void);
 
 /*
