@@ -40,7 +40,6 @@
 #include <linux/keyboard.h>
 #include <linux/vt.h>
 
-#define SIZE(array) (sizeof(array) / sizeof(array[0]))
 #define BITS_PER_ULDAT (sizeof(uldat) * 8)
 
 #include "kbd_bitops.h"
@@ -51,7 +50,7 @@
 #define VC_CRLF 3   /* 0 - enter sends CR, 1 - enter sends CRLF */
 #define VC_META 4   /* 0 - meta, 1 - meta=prefix with ESC */
 
-#define vc_kbd_mode(kbd, bit) (lrawkbd_config >> (bit)&1)
+#define vc_kbd_mode(kbd, bit) (lrawkbd_config >> (bit) & 1)
 #define set_vc_kbd_mode(kbd, bit) (lrawkbd_config |= 1L << (bit))
 #define clr_vc_kbd_mode(kbd, bit) (lrawkbd_config &= ~(1L << (bit)))
 #define chg_vc_kbd_mode(kbd, bit) (lrawkbd_config ^= 1L << (bit))
@@ -60,7 +59,7 @@
 #define VC_NUMLOCK 1   /* numeric lock led mode */
 #define VC_CAPSLOCK 2  /* capslock led mode */
 
-#define vc_kbd_led(kbd, bit) (lrawkbd_leds >> (bit)&1)
+#define vc_kbd_led(kbd, bit) (lrawkbd_leds >> (bit) & 1)
 #define set_vc_kbd_led(kbd, bit) set_lights(lrawkbd_leds |= 1L << (bit))
 #define clr_vc_kbd_led(kbd, bit) set_lights(lrawkbd_leds &= ~(1L << (bit)))
 #define chg_vc_kbd_led(kbd, bit) set_lights(lrawkbd_leds ^= 1L << (bit))
@@ -121,10 +120,10 @@ static void applkey(char key, byte mode) {
   puts_queue(buf);
 }
 
-static void noop_fn(void) {
+static void noop_fn() {
 }
 
-static void enter(void) {
+static void enter() {
   if (diacr) {
     put_queue(diacr);
     diacr = 0;
@@ -134,17 +133,17 @@ static void enter(void) {
     put_queue(10);
 }
 
-static void caps_toggle(void) {
+static void caps_toggle() {
   if (!rep)
     chg_vc_kbd_led(kbd, VC_CAPSLOCK);
 }
 
-static void caps_on(void) {
+static void caps_on() {
   if (!rep)
     set_vc_kbd_led(kbd, VC_CAPSLOCK);
 }
 
-static void hold(void) {
+static void hold() {
 #if 1
   if (!rep)
     chg_vc_kbd_led(kbd, VC_SCROLLOCK);
@@ -170,66 +169,64 @@ static void hold(void) {
  * Bind this to NumLock if you prefer that the NumLock key always
  * changes the NumLock flag.
  */
-static void bare_num(void) {
-  if (!rep)
+static void bare_num() {
+  if (!rep) {
     chg_vc_kbd_led(kbd, VC_NUMLOCK);
+  }
 }
 
-static void num(void) {
-  if (vc_kbd_mode(kbd, VC_APPLIC))
+static void num() {
+  if (vc_kbd_mode(kbd, VC_APPLIC)) {
     applkey('P', 1);
-  else
+  } else {
     bare_num();
+  }
 }
 
 static void set_console(byte nr) {
-  ioctl(tty_fd, VT_ACTIVATE, nr & 63);
+  tty_driver *self = ttydriver(lrawkbd_hw);
+  ioctl(self->tty_fd, VT_ACTIVATE, nr & 63);
 
   /* we will receive SIGUSR1 if kernel needs our permission to switch */
 }
 
-static udat get_console_bitmap(void) {
+static udat get_console_bitmap() {
   struct vt_stat vs;
-#if 0
-    struct vt_stat {
-        udat v_active;        /* active vt */
-        udat v_signal;        /* signal to send */
-        udat v_state;                /* vt bitmask */
-    };
-#endif
+  tty_driver *self = ttydriver(lrawkbd_hw);
 
-  if (ioctl(tty_fd, VT_GETSTATE, &vs) >= 0)
+  if (ioctl(self->tty_fd, VT_GETSTATE, &vs) >= 0) {
     return vs.v_state & ~1; /* skip tty0 */
+  }
   return 0xFFFE;
 }
 
 #define b16 (sizeof(udat) * 8)
 
 static void set_console_ifthere(uldat k) {
-  if (k >= b16 /* get_console_bitmap() cannot tell beyond b16 */ || get_console_bitmap() >> k & 1) {
-
+  if (k >= b16 /* get_console_bitmap() cannot tell beyond b16 */
+      || get_console_bitmap() >> k & 1) {
     set_console(k);
   }
 }
 
 /* get the last used console */
-static uldat get_console_last(void) {
+static uldat get_console_last() {
   uldat b = get_console_bitmap();
   uldat k = 0;
-
-  while (b >>= 1)
+  while (b >>= 1) {
     k++;
-
+  }
   return k;
 }
 
-static void set_console_last(void) {
+static void set_console_last() {
   /* switch to the last used console, ChN */
   set_console(get_console_last());
 }
 
-static void decr_console(void) {
-  uldat k = tty_number - 1;
+static void decr_console() {
+  tty_driver *self = ttydriver(lrawkbd_hw);
+  uldat k = self->tty_number - 1;
   udat b;
 
   if (k >= b16) /* get_console_bitmap() cannot tell beyond b16 */
@@ -240,16 +237,19 @@ static void decr_console(void) {
 
   while (b && !(b & 1 << (b16 - 1))) {
     b <<= 1;
-    if (--k >= b16)
+    if (--k >= b16) {
       k = b16 - 1;
+    }
   }
 
-  if (b)
+  if (b) {
     set_console(k);
+  }
 }
 
-static void incr_console(void) {
-  uldat k = tty_number + 1;
+static void incr_console() {
+  tty_driver *self = ttydriver(lrawkbd_hw);
+  uldat k = self->tty_number + 1;
   udat b;
 
   k &= (udat)~0; /* get_console_bitmap() cannot tell beyond b16 */
@@ -262,33 +262,35 @@ static void incr_console(void) {
     k = (k + 1) & (udat)~0;
   }
 
-  if (b)
+  if (b) {
     set_console(k);
+  }
 }
 
-static void send_intr(void) {
+static void send_intr() {
 #if 0
-    tty_break();
+  tty_break();
 #endif
 }
 
-static void scroll_forw(void) { /* FIXME: finish this! */
+static void scroll_forw() { /* FIXME: finish this! */
 }
 
-static void scroll_back(void) { /* FIXME: finish this! */
+static void scroll_back() { /* FIXME: finish this! */
 }
 
-static void boot_it(void) { /* HW->NeedHW |= NEEDPanicHW, NeedHW |= NEEDPanicHW; */
+static void boot_it() { /* hw->NeedHW |= NeedPanicHW, NeedHW |= NeedPanicHW; */
 }
 
-static void compose(void) {
+static void compose() {
   dead_key_next = 1;
 }
 
-static void spawn_console(void) { /* no way to reproduce this from user level */
+static void spawn_console() {
+  /* no way to reproduce this from user level */
 }
 
-static void SAK(void) {
+static void SAK() {
   /*
    * SAK should also work in all raw modes and reset
    * them properly.
@@ -298,15 +300,20 @@ static void SAK(void) {
 }
 
 static udat do_ignore(byte value, byte up_flag) {
+  (void)value;
+  (void)up_flag;
   return TW_Null;
 }
 
-static void do_null(void) {
+static void do_null() {
   compute_shiftstate();
 }
 
 static udat do_lowercase(byte value, byte up_flag) {
-  printk("twin: hw/linux/kbd_low.h: do_lowercase() called - kernel bug!\n");
+  (void)value;
+  (void)up_flag;
+  log(ERROR)
+      << "twin: hw/hw_tty_linux/kbd_raw1.h: do_lowercase() called. This should not happen!\n";
   return TW_Null;
 }
 
@@ -321,6 +328,7 @@ static char handle_diacr(char ch) {
   static struct kbdiacrs accent_tables;
   static byte accent_tables_init;
 
+  tty_driver *self = ttydriver(lrawkbd_hw);
   struct kbdiacr *accent_table;
   char d = diacr;
   unsigned i;
@@ -329,7 +337,7 @@ static char handle_diacr(char ch) {
 
   if (!accent_tables_init) {
     accent_tables_init = ttrue;
-    ioctl(tty_fd, KDGKBDIACR, &accent_tables);
+    ioctl(self->tty_fd, KDGKBDIACR, &accent_tables);
   }
 
   accent_table = accent_tables.kbdiacr;
@@ -342,7 +350,7 @@ static char handle_diacr(char ch) {
   if (ch == ' ' || ch == d)
     return d;
 
-  KeyboardEventCommon(d, get_shiftstate_tw(), 1, &d);
+  KeyboardEventCommon(lrawkbd_hw, d, get_shiftstate_tw(), 1, &d);
 
   return ch;
 }
@@ -444,7 +452,7 @@ static udat do_fn(byte value, byte up_flag) {
     return TW_Null;
   puts_queue(get_kbsentry(value));
 
-  return value < SIZE(map_FN) ? map_FN[value] : 0;
+  return value < N_OF(map_FN) ? map_FN[value] : 0;
 }
 
 static udat map_PAD[] = {
@@ -469,7 +477,7 @@ static udat map_PAD[] = {
     /* K_PPARENL */ TW_Null,
     /* K_PPARENR */ TW_Null,
 };
-enum { map_PAD_len = sizeof(map_PAD) / sizeof(map_PAD[0]) };
+enum { map_PAD_len = N_OF(map_PAD) };
 
 static udat do_pad(byte value, byte up_flag) {
   static const char *const pad_chars = "0123456789+-*/\015,.?()";
@@ -532,7 +540,7 @@ static udat map_CUR[] = {
     /* K_RIGHT */ TW_Right,
     /* K_UP */ TW_Up,
 };
-enum { map_CUR_len = sizeof(map_CUR) / sizeof(map_CUR[0]) };
+enum { map_CUR_len = N_OF(map_CUR) };
 
 static udat do_cur(byte value, byte up_flag) {
   static const char *const cur_chars = "BDCA";
@@ -555,7 +563,7 @@ static udat map_SHIFT[] = {
     /* K_CTRLR */ TW_Control_R,
     /* K_CAPSSHIFT*/ TW_Shift_R,
 };
-enum { map_SHIFT_len = sizeof(map_SHIFT) / sizeof(map_SHIFT[0]) };
+enum { map_SHIFT_len = N_OF(map_SHIFT) };
 
 static udat do_shift(byte value, byte up_flag) {
   ldat old_state = lrawkbd_shiftstate;
@@ -587,25 +595,25 @@ static udat do_shift(byte value, byte up_flag) {
   /* kludge */
   if (up_flag && lrawkbd_shiftstate != old_state && npadch != -1) {
     char ch = npadch & 0xff;
-    KeyboardEventCommon(ch, get_shiftstate_tw(), 1, &ch);
+    KeyboardEventCommon(lrawkbd_hw, ch, get_shiftstate_tw(), 1, &ch);
     npadch = -1;
   }
 
-  return (up_flag || value >= map_SHIFT_len) ? TW_Null : map_SHIFT[value];
+  return (up_flag || value >= map_SHIFT_len) ? (udat)TW_Null : map_SHIFT[value];
 }
 
 /* called after returning from RAW mode or when changing consoles -
  recompute k_down[] and shift_state from key_down[] */
 /* maybe called when keymap is undefined, so that shiftkey release is seen */
-static int compute_shiftstate(void) {
+static int compute_shiftstate() {
   uldat i, j, k;
   udat sym, val;
 
   lrawkbd_shiftstate = 0;
-  for (i = 0; i < SIZE(k_down); i++)
+  for (i = 0; i < N_OF(k_down); i++)
     k_down[i] = 0;
 
-  for (i = 0; i < SIZE(key_down); i++) {
+  for (i = 0; i < N_OF(key_down); i++) {
     if (key_down[i]) { /* skip this word if not a single bit on */
       k = i * BITS_PER_ULDAT;
       for (j = 0; j < BITS_PER_ULDAT; j++, k++) {
@@ -625,7 +633,7 @@ static int compute_shiftstate(void) {
   return lrawkbd_shiftstate;
 }
 
-static udat get_shiftstate_tw(void) {
+static udat get_shiftstate_tw() {
   return (lrawkbd_shiftstate & (1 << KG_SHIFT | 1 << KG_SHIFTL | 1 << KG_SHIFTR) ? KBD_SHIFT_FL
                                                                                  : 0) |
          (lrawkbd_shiftstate & (1 << KG_CTRL | 1 << KG_CTRLL | 1 << KG_CTRLR) ? KBD_CTRL_FL : 0) |
@@ -754,26 +762,25 @@ static udat do_spec(byte value, byte up_flag) {
 
 typedef udat (*k_hand)(byte value, byte up_flag);
 
-static k_hand key_handler[] = {
-    do_latin, do_fn,    do_spec, do_pad,       do_dead,  do_cons,  do_cur,    do_shift,
-
-    do_meta,  do_ascii, do_lock, do_lowercase, do_slock, do_dead2, do_ignore, do_ignore};
+static k_hand key_handler[] = {do_latin, do_fn,    do_spec,   do_pad,   do_dead, do_cons,
+                               do_cur,   do_shift, do_meta,   do_ascii, do_lock, do_lowercase,
+                               do_slock, do_dead2, do_ignore, do_ignore};
 
 /* maximum values each key_handler can handle */
 static const byte key_handler_maxval[] = {255,
                                           255,
-                                          SIZE(map_SPEC) - 1,
-                                          SIZE(map_PAD) - 1,
-                                          SIZE(map_DIACR) - 1,
+                                          N_OF(map_SPEC) - 1,
+                                          N_OF(map_PAD) - 1,
+                                          N_OF(map_DIACR) - 1,
                                           255,
-                                          SIZE(map_CUR) - 1,
-                                          SIZE(map_SHIFT) - 1,
+                                          N_OF(map_CUR) - 1,
+                                          N_OF(map_SHIFT) - 1,
 
                                           255,
                                           NR_ASCII - 1,
-                                          SIZE(map_LOCK) - 1,
+                                          N_OF(map_LOCK) - 1,
                                           255,
-                                          SIZE(map_SLOCK) - 1,
+                                          N_OF(map_SLOCK) - 1,
                                           255,
                                           0,
                                           0};
@@ -786,14 +793,16 @@ static void set_lights(int lights) {
         (lights & 1<<VC_NUMLOCK   ? LED_NUM : 0) |
         (lights & 1<<VC_CAPSLOCK  ? LED_CAP : 0);
 #endif
-  ioctl(tty_fd, KDSETLED, lights);
+  tty_driver *self = ttydriver(lrawkbd_hw);
+
+  ioctl(self->tty_fd, KDSETLED, lights);
 }
 
 #define lrawkbd_KEYMAPS_N (1 << (NR_SHIFT <= 8 * sizeof(byte) ? NR_SHIFT : 8 * sizeof(byte)))
 
 static udat *lrawkbd_keymaps[lrawkbd_KEYMAPS_N];
 
-static void lrawkbd_FreeKeymaps(void) {
+TW_ATTR_HIDDEN void tty_driver::lrawkbdFreeKeymaps() {
   udat *keymap;
   ldat table;
   for (table = 0; table < lrawkbd_KEYMAPS_N; table++) {
@@ -804,18 +813,21 @@ static void lrawkbd_FreeKeymaps(void) {
   }
 }
 
-static void lrawkbd_LoadKeymaps(void) {
+TW_ATTR_HIDDEN bool tty_driver::lrawkbdLoadKeymaps() {
   struct kbentry ke;
+  tty_driver *self = ttydriver(lrawkbd_hw);
   ldat table, keycode;
 
   /*
    * set UNICODE keyboard mode.
    * needed to get accurate results from ioctl(tty_fd, KDGKBENT, ...)
    *
-   * keyboard mode will be overridden in lrawkbd_SetKeyboard(),
+   * keyboard mode will be overridden in lrawkbdSetKeyboard(),
    * no need to restore it here.
    */
-  ioctl(tty_fd, KDSKBMODE, K_UNICODE);
+  if (ioctl(self->tty_fd, KDSKBMODE, K_UNICODE) < 0) {
+    return false;
+  }
 
   for (table = 0; table < lrawkbd_KEYMAPS_N; table++) {
     for (keycode = 0; keycode < 0x80; keycode++) {
@@ -824,11 +836,12 @@ static void lrawkbd_LoadKeymaps(void) {
       ke.kb_index = keycode;
       ke.kb_value = 0;
 
-      if (ioctl(tty_fd, KDGKBENT, &ke) == 0) {
+      if (ioctl(self->tty_fd, KDGKBENT, &ke) == 0) {
 
 #ifdef DEBUG_HW_TTY_LRAWKBD
-        printk("\t... ioctl(tty_fd = %d, KDGKBENT, {table = 0x%X, index = 0x%X} ) = 0x%X\n",
-               (int)tty_fd, table, keycode, (int)ke.kb_value);
+        log(INFO) << "\t... ioctl(tty_fd = " << (int)tty_fd << ", KDGKBENT, {table = 0x"
+                  << hex(table) << ", index = 0x" << hex(keycode) << "} ) = 0x"
+                  << hex((int)ke.kb_value) << "\n";
 #endif
 
         if (keycode == 0 && ke.kb_value == K_NOSUCHMAP)
@@ -841,6 +854,7 @@ static void lrawkbd_LoadKeymaps(void) {
       }
     }
   }
+  return true;
 }
 
 static udat get_kbentry(byte keycode, byte table) {
@@ -848,21 +862,23 @@ static udat get_kbentry(byte keycode, byte table) {
     udat *keymap = lrawkbd_keymaps[table];
     if (keymap != NULL) {
 #ifdef DEBUG_HW_TTY_LRAWKBD
-      printk("\tget_kbentry(table = 0x%X, keycode = 0x%X) = 0x%X\n", (int)table, (int)keycode,
-             (int)keymap[keycode & 0x7F]);
+      log(INFO) << "\tget_kbentry(table = 0x" << hex(int(table)) << ", keycode = 0x"
+                << hex(int(keycode)) << ") = 0x" << hex(int(keymap[keycode & 0x7F])) << "\n";
 #endif
       return keymap[keycode & 0x7F];
     }
-  } else
-    printk("internal error! invalid arguments in get_kbentry(keycode = 0x%X, table = 0x%X): table "
-           "must be < 0x%X\n",
-           (int)keycode, (int)table, (int)lrawkbd_KEYMAPS_N);
-
+  } else {
+    log(ERROR) << "internal error! invalid arguments in get_kbentry(keycode = 0x"
+               << hex(int(keycode)) << ", table = 0x" << hex(int(table)) << "): table must be < 0x"
+               << hex(int(lrawkbd_KEYMAPS_N)) << "\n";
+  }
   return K_NOSUCHMAP;
 }
 
 static const char *get_kbsentry(byte keysym) {
   static struct kbsentry ks;
+  tty_driver *self = ttydriver(lrawkbd_hw);
+
   /*
    struct kbsentry {
       unsigned char kb_func;
@@ -873,7 +889,7 @@ static const char *get_kbsentry(byte keysym) {
   ks.kb_func = keysym;
   ks.kb_string[0] = '\0';
 
-  ioctl(tty_fd, KDGKBSENT, &ks);
+  ioctl(self->tty_fd, KDGKBSENT, &ks);
 
   return (const char *)ks.kb_string;
 }
@@ -883,15 +899,15 @@ static void dump_key(udat twk, ldat keysym, byte *s, uldat len) {
   uldat f;
   byte c;
 
-  printk("\tTWsym = 0x%X, ksym = 0x%X, `", (int)twk, (int)keysym);
+  log(INFO) << "\tTWsym = 0x" << hex(int(twk)) << ", ksym = 0x" << hex(int(keysym)) << ", `";
 
   for (f = 0; (c = s[f]) && f < len; f++) {
     if (c >= ' ' && c <= '~')
-      printk("%c", (int)c);
+      log(INFO) << Chars(&c, 1);
     else
-      printk("\\x%02X", (int)c);
+      log(INFO) << "\\x" << hex((unsigned char)c);
   }
-  printk("'\n");
+  log(INFO) << "'\n";
 }
 #endif
 
@@ -925,7 +941,7 @@ static udat handle_keycode(byte keycode, byte up) {
     }
 
     val = KVAL(keysym);
-    if (type < SIZE(key_handler_maxval) && val <= key_handler_maxval[type])
+    if (type < N_OF(key_handler_maxval) && val <= key_handler_maxval[type])
       twk = (*key_handler[type])(val, up_flag);
 
     if (type != KT_SLOCK)
@@ -939,7 +955,8 @@ static udat handle_keycode(byte keycode, byte up) {
   return twk;
 }
 
-static udat lrawkbd_LookupKey(udat *ShiftFlags, byte *slen, char *s, byte *retlen, char **ret) {
+TW_ATTR_HIDDEN udat tty_driver::lrawkbdLookupKey(udat *ShiftFlags, byte *slen, char *s,
+                                                 byte *retlen, char **ret) {
   udat twk;
   byte k = (byte)*s;
 
