@@ -10,17 +10,9 @@
 /*
  *             W A R N I N G  :
  *
- * the ORDER of the IS() macros in this file is CAREFULLY chosen
+ * the ORDER of the IS() macros in this file is carefully chosen
  * so that the corresponding XK_* KeySym (from <X11/keysymdef.h>)
- * have continuously GROWING values (i.e. are sorted).
- *
- * If you need to add more entries in this file,
- * look at the include file <X11/keysymdef.h> and put new entries
- * at the exact position to preserve sorting.
- *
- * If you mess up this file, compiling `twin' with X11 support
- * will result in a non-working executable.
- *
+ * have increasing values (i.e. are sorted).
  */
 
 IS(BackSpace, 1, "\x7F")
@@ -43,7 +35,7 @@ IS(Down, 3, "\x1B[B")
 IS(Prior, 4, "\x1B[5~")
 IS(Next, 4, "\x1B[6~")
 IS(End, 4, "\x1B[4~")
-IS(Begin, 4, "\x1B[1~") /* as Home */
+IS(Begin, 4, "\x1B[E") /* as KP_Begin */
 IS(Select, 0, "")
 IS(Print, 0, "")
 IS(Execute, 0, "")
@@ -57,77 +49,78 @@ IS(Cancel, 0, "")
 IS(Help, 0, "")
 IS(Break, 0, "")
 /*IS(Mode_switch,0, "")*/ /* I don't want to receive this, it's AltGr */
-IS(Num_Lock, 3, "\x1BOP")
+IS(Num_Lock, 3, "\x1BON") /* xterm produces nothing,
+                           * vt220 produces "\x1BOP" duplicate of xterm F1 */
 
 IS(KP_Space, 1, " ")
 IS(KP_Tab, 1, "\x09")
 IS(KP_Enter, 3, "\x1BOM")
 
-IS(KP_F1, 4, "\x1B[[A") /* as F1 */
-IS(KP_F2, 4, "\x1B[[B") /* as F2 */
-IS(KP_F3, 4, "\x1B[[C") /* as F3 */
-IS(KP_F4, 4, "\x1B[[D") /* as F4 */
+IS(KP_F1, 4, "\x1BOP") /* as F1 */
+IS(KP_F2, 4, "\x1BOQ") /* as F2 */
+IS(KP_F3, 4, "\x1BOR") /* as F3 */
+IS(KP_F4, 4, "\x1BOS") /* as F4 */
 
 /* these are duplicates of KP_[0..9] but have different X11 keysim */
 #if defined(XK_KP_Home) && (XK_KP_Home != XK_KP_7)
-IS(KP_Home, 3, "\x1BOw")
+IS(KP_Home, 3, "\x1B[H") /* as KP_7 */
 #endif
 #if defined(XK_KP_Left) && (XK_KP_Left != XK_KP_4)
-IS(KP_Left, 3, "\x1BOt")
+IS(KP_Left, 3, "\x1B[D") /* as KP_4 */
 #endif
 #if defined(XK_KP_Up) && (XK_KP_Up != XK_KP_8)
-IS(KP_Up, 3, "\x1BOx")
+IS(KP_Up, 3, "\x1B[A") /* as KP_8 */
 #endif
 #if defined(XK_KP_Right) && (XK_KP_Right != XK_KP_6)
-IS(KP_Right, 3, "\x1BOv")
+IS(KP_Right, 3, "\x1B[C") /* as KP_6 */
 #endif
 #if defined(XK_KP_Down) && (XK_KP_Down != XK_KP_2)
-IS(KP_Down, 3, "\x1BOr")
+IS(KP_Down, 3, "\x1B[B") /* as KP_2 */
 #endif
 #if defined(XK_KP_Prior) && (XK_KP_Prior != XK_KP_9)
-IS(KP_Prior, 3, "\x1BOy")
+IS(KP_Prior, 3, "\x1B[5~") /* as KP_9 */
 #endif
 #if defined(XK_KP_Next) && (XK_KP_Next != XK_KP_3)
-IS(KP_Next, 3, "\x1BOs")
+IS(KP_Next, 3, "\x1B[6~") /* as KP_3 */
 #endif
 #if defined(XK_KP_End) && (XK_KP_End != XK_KP_1)
-IS(KP_End, 3, "\x1BOq")
+IS(KP_End, 3, "\x1B[F") /* as KP_1 */
 #endif
 #if defined(XK_KP_Begin) && (XK_KP_Begin != XK_KP_5)
-IS(KP_Begin, 3, "\x1BOu")
+IS(KP_Begin, 3, "\x1B[E") /* as KP_5 */
 #endif
 #if defined(XK_KP_Insert) && (XK_KP_Insert != XK_KP_0)
-IS(KP_Insert, 3, "\x1BOp")
+IS(KP_Insert, 3, "\x1B[[2~") /* as KP_0 */
 #endif
 #if defined(XK_KP_Delete) && (XK_KP_Delete != XK_KP_Decimal)
-IS(KP_Delete, 3, "\x1BOn")
+IS(KP_Delete, 3, "\x1B[[3~") /* as KP_Decimal */
 #endif
 
-IS(KP_Multiply, 3, "\x1BOR")
-IS(KP_Add, 3, "\x1BOl")
+IS(KP_Multiply, 3, "\x1BOj")
+IS(KP_Add, 3, "\x1BOk")
 IS(KP_Separator, 3, "\x1BOm")
-IS(KP_Subtract, 3, "\x1BOS")
-IS(KP_Decimal, 3, "\x1BOn") /* as KP_Delete */
-IS(KP_Divide, 3, "\x1BOQ")
+IS(KP_Subtract, 3, "\x1BOm")
+IS(KP_Decimal, 3, "\x1B[[3~")
+IS(KP_Divide, 3, "\x1BOo")
 
-IS(KP_0, 3, "\x1BOp")
-IS(KP_1, 3, "\x1BOq")
-IS(KP_2, 3, "\x1BOr")
-IS(KP_3, 3, "\x1BOs")
-IS(KP_4, 3, "\x1BOt")
-IS(KP_5, 3, "\x1BOu")
-IS(KP_6, 3, "\x1BOv")
-IS(KP_7, 3, "\x1BOw")
-IS(KP_8, 3, "\x1BOx")
-IS(KP_9, 3, "\x1BOy")
+IS(KP_0, 4, "\x1B[2~")
+IS(KP_1, 3, "\x1B[F")
+IS(KP_2, 3, "\x1B[B")
+IS(KP_3, 4, "\x1B[6~")
+IS(KP_4, 3, "\x1B[D")
+IS(KP_5, 3, "\x1B[E")
+IS(KP_6, 3, "\x1B[C")
+IS(KP_7, 3, "\x1B[H")
+IS(KP_8, 3, "\x1B[A")
+IS(KP_9, 4, "\x1B[5~")
 
-IS(KP_Equal, 3, "\x1BOo")
+IS(KP_Equal, 3, "\x1BOX")
 
-IS(F1, 4, "\x1B[[A")
-IS(F2, 4, "\x1B[[B")
-IS(F3, 4, "\x1B[[C")
-IS(F4, 4, "\x1B[[D")
-IS(F5, 4, "\x1B[[E")
+IS(F1, 4, "\x1BOP")
+IS(F2, 4, "\x1BOQ")
+IS(F3, 4, "\x1BOR")
+IS(F4, 4, "\x1BOS")
+IS(F5, 4, "\x1B[15~")
 IS(F6, 5, "\x1B[17~")
 IS(F7, 5, "\x1B[18~")
 IS(F8, 5, "\x1B[19~")
