@@ -1,5 +1,5 @@
 /*
- *  rcrun.c  --  virtual machine to run byte-compiled ~/.twinrc code
+ *  rcrun.c  --  virtual machine to run byte-compiled configuration file ~/.config/twin/twinrc
  *
  *  Copyright (C) 2001 by Massimiliano Ghilardi
  *
@@ -19,14 +19,17 @@
 #include "obj/id.h" // Obj2Id()
 #include "resize.h"
 #include "util.h"
+#include "tterm.h" // OpenTerm()
 #include "wm.h"
 
+#include "common.h"
 #include "dl.h"
 #include "extreg.h"
-#include "methods.h"
 #include "hw.h"
-#include "common.h"
 #include "hw_multi.h"
+#include "log.h"
+#include "methods.h"
+#include "unaligned.h"
 
 #include "rctypes.h"
 #include "rcparse_tab.hpp"
@@ -42,21 +45,23 @@
 #include <Tutf/Tutf.h>
 #include <Tutf/Tutf_defs.h>
 
-#define MAX_RUNCYCLE 1024 /* kill a queue after this number of steps */
+enum {
+  MAX_RUNCYCLE = 1024, /* kill a queue after this number of steps */
 
-#define MAX_RUNSTACK 28
-typedef struct run run;
+  MAX_RUNSTACK = 28,
+};
 
-struct run {
+class run {
+public:
   run *next;
-  uldat W;     /* the current widget (id) */
+  uldat W;     /* the current Twidget (id) */
   uldat depth; /* index of last used stack element;
                 * stack[depth] is the current instruction */
   uldat cycle;
   union {
     timevalue WakeUp;
     cstr Name;
-  } SW; /* what we are waiting for: sleep timeout or widget map */
+  } SW; /* what we are waiting for: sleep timeout or Twidget map */
 
   wm_ctx C; /* event that generated the run queue */
   node stack[MAX_RUNSTACK];
@@ -247,17 +252,17 @@ static node RCFindMouseBind(ldat code, ldat ctx) {
   return NULL;
 }
 
-trune *RCFindBorderPattern(window W, byte Border) {
+trune *RCFindBorderPattern(Twindow w, byte border) {
   node l;
 
-  if (!W)
+  if (!w)
     return NULL;
 
   for (l = BorderList; l; l = l->next) {
-    if ((l->x.f.flag == FL_INACTIVE) == Border && wildcard_match(l->name, W->Name))
+    if ((l->x.f.flag == FL_INACTIVE) == border && wildcard_match(l->name, w->Name))
       break;
   }
-  return W->BorderPattern[Border] = l ? (trune *)l->data : NULL;
+  return w->BorderPattern[border] = l ? l->runes : NULL;
 }
 
 inline void RCRemove(run **p) {
@@ -347,86 +352,90 @@ static ldat applyflagy(node n) {
   return y * n->x.f.b;
 }
 
-static window RCFindWindowName(cstr name) {
+static Twindow RCFindWindowName(cstr name) {
   uldat len = strlen(name);
-  window W;
-  screen S = All->FirstScreen;
+  Twindow w;
+  Tscreen screen = All->Screens.First;
 
-  while (S) {
+  while (screen) {
     /* search among mapped windows */
-    W = (window)S->FirstW;
-    while (W) {
-      if (IS_WINDOW(W) && W->NameLen >= 0 && (udat)W->NameLen == len && !memcmp(W->Name, name, len))
-        return W;
-      W = (window)W->Next;
+    w = (Twindow)screen->Widgets.First;
+    while (w) {
+      if (IS_WINDOW(w) && w->NameLen >= 0 && (udat)w->NameLen == len &&
+          !memcmp(w->Name, name, len)) {
+        return w;
+      }
+      w = (Twindow)w->Next;
     }
-    S = S->Next;
+    screen = screen->NextScreen();
   }
   return NULL;
 }
 
-static screen RCFindScreenName(cstr name) {
+static Tscreen RCFindScreenName(cstr name) {
   uldat len = strlen(name);
-  screen S = All->FirstScreen;
-  while (S) {
-    if (S->NameLen >= 0 && (udat)S->NameLen == len && !memcmp(S->Name, name, len))
+  Tscreen screen = All->Screens.First;
+  while (screen) {
+    if (screen->NameLen >= 0 && (udat)screen->NameLen == len && !memcmp(screen->Name, name, len))
       break;
   }
-  return S;
+  return screen;
 }
 
-inline widget RCCheck4WidgetId(run *r) {
-  widget W;
-  if (!(W = (widget)Id2Obj(widget_magic_id, r->W)) || !W->Parent || !IS_SCREEN(W->Parent))
+inline Twidget RCCheck4WidgetId(run *r) {
+  Twidget w;
+  if (!(w = (Twidget)Id2Obj(Twidget_class_byte, r->W)) || !w->Parent || !IS_SCREEN(w->Parent))
 
     r->W = NOID;
-  return W;
+  return w;
 }
 
-#define Snext 0
-#define Sfunc 1
-#define Sbody 2
-#define Ssleep 3
-#define Swait 4
-#define Sinter 5
-#define Serr 6
+enum {
+  Snext = 0,
+  Sfunc = 1,
+  Sbody = 2,
+  Ssleep = 3,
+  Swait = 4,
+  Sinter = 5,
+  Serr = 6,
+};
 
-inline widget ForwardWindow(widget W) {
-  while (W) {
-    if (IS_WINDOW(W))
-      return W;
-    W = W->Next;
+inline Twidget ForwardWindow(Twidget w) {
+  while (w) {
+    if (IS_WINDOW(w))
+      return w;
+    w = w->Next;
   }
-  return W;
+  return w;
 }
 
-inline widget BackwardWindow(widget W) {
-  while (W) {
-    if (IS_WINDOW(W))
-      return W;
-    W = W->Prev;
+inline Twidget BackwardWindow(Twidget w) {
+  while (w) {
+    if (IS_WINDOW(w))
+      return w;
+    w = w->Prev;
   }
-  return W;
+  return w;
 }
 
-inline screen ScreenOf(widget W) {
-  widget P;
-  return W && (P = W->Parent) && IS_SCREEN(P) ? (screen)P : (screen)0;
+inline Tscreen ScreenOf(Twidget w) {
+  Twidget P;
+  return w && (P = w->Parent) && IS_SCREEN(P) ? (Tscreen)P : (Tscreen)0;
 }
 
 /* run the specified queue */
 static byte RCSteps(run *r) {
-  widget W, SkipW;
-  screen S;
+  Twidget w, SkipW;
+  Tscreen screen;
   wm_ctx *C;
   node n, f;
   ldat flag;
   const cstr *argv;
-  byte state, ret;
+  byte state = Snext, ret;
   int nfd;
 
-  W = RCCheck4WidgetId(r);
-  S = ScreenOf(W);
+  w = RCCheck4WidgetId(r);
+  screen = ScreenOf(w);
   C = &r->C;
   n = r->stack[r->depth];
 
@@ -437,9 +446,9 @@ static byte RCSteps(run *r) {
       switch (n->id) {
 
       case EXEC:
-        if (flag_secure)
-          printk(flag_secure_msg);
-        else
+        if (flag_secure) {
+          log(ERROR) << flag_secure_msg;
+        } else {
           switch (fork()) {
           case -1: /* error */
             break;
@@ -459,6 +468,7 @@ static byte RCSteps(run *r) {
           default: /* parent */
             break;
           }
+        }
         break;
 
       case EXECTTY:
@@ -467,13 +477,13 @@ static byte RCSteps(run *r) {
          * it goes in a msg_map queued in the WM_MsgPort,
          * so we have the time to reach the corresponding WAIT here
          */
-        Ext(Term, Open)(n->x.v.argv[0], n->x.v.argv);
+        OpenTerm(n->x.v.argv[0], n->x.v.argv);
         break;
 
       case INTERACTIVE:
-        C->W = W;
-        if (!C->Screen && !(C->Screen = S))
-          C->Screen = All->FirstScreen;
+        C->W = w;
+        if (!C->Screen && !(C->Screen = screen))
+          C->Screen = All->Screens.First;
 
         ret = tfalse;
         switch (n->x.f.flag) {
@@ -504,20 +514,22 @@ static byte RCSteps(run *r) {
         ActivateCtx(C, state_menu);
         break;
       case MODULE:
-        if (n->x.f.a == -1)
-          n->x.f.a = DlName2Code(n->name);
-        if (n->x.f.flag == FL_ON)
-          DlLoad(n->x.f.a);
-        else
-          DlUnload(n->x.f.a);
+        if (n->x.f.a == -1) {
+          n->x.f.a = DlName2Code(Chars::from_c(n->name));
+        }
+        if (n->x.f.flag == FL_ON) {
+          DlLoad((IdSo)n->x.f.a);
+        } else {
+          DlUnload((IdSo)n->x.f.a);
+        }
         break;
       case MOVE:
-        if (W && IS_WINDOW(W))
-          DragWindow((window)W, applyflagx(n), applyflagy(n));
+        if (w && IS_WINDOW(w))
+          DragWindow((Twindow)w, applyflagx(n), applyflagy(n));
         break;
       case MOVESCREEN:
-        if (S && S != All->FirstScreen)
-          S->Focus();
+        if (screen && screen != All->Screens.First)
+          screen->Focus();
         DragFirstScreen(applyflagx(n), applyflagy(n));
         break;
       case NOP:
@@ -527,31 +539,31 @@ static byte RCSteps(run *r) {
         return Serr;
         break;
       case RESIZE:
-        if (W && IS_WINDOW(W)) {
+        if (w && IS_WINDOW(w)) {
           dat x = applyflagx(n), y = applyflagy(n);
           if (n->x.f.plus_minus == 0)
-            x = n->x.f.a - W->XWidth + 2 * !(W->Flags & WINDOWFL_BORDERLESS);
+            x = n->x.f.a - w->XWidth + 2 * !(w->Flags & WINDOWFL_BORDERLESS);
           if (n->x.f.flag == 0)
-            y = n->x.f.b - W->YWidth + 2 * !(W->Flags & WINDOWFL_BORDERLESS);
-          ResizeRelWindow((window)W, x, y);
-          Check4Resize((window)W); /* send MSG_WINDOW_CHANGE and realloc(W->Contents) */
+            y = n->x.f.b - w->YWidth + 2 * !(w->Flags & WINDOWFL_BORDERLESS);
+          ResizeRelWindow((Twindow)w, x, y);
+          Check4Resize((Twindow)w); /* send MSG_WINDOW_CHANGE and realloc(w->Contents) */
         }
         break;
       case RESIZESCREEN:
-        if (S && S != All->FirstScreen)
-          S->Focus();
+        if (screen && screen != All->Screens.First)
+          screen->Focus();
         ResizeFirstScreen(applyflagx(n));
         break;
       case SCROLL:
-        if (W && IS_WINDOW(W))
-          ScrollWindow((window)W, applyflagx(n), applyflagy(n));
+        if (w && IS_WINDOW(w))
+          ScrollWindow((Twindow)w, applyflagx(n), applyflagy(n));
         break;
       case SENDTOSCREEN:
-        if (W && IS_WINDOW(W) && S && n->name) {
-          screen Screen = RCFindScreenName(n->name);
-          if (S != Screen) {
-            W->UnMap();
-            W->Map((widget)Screen);
+        if (w && IS_WINDOW(w) && screen && n->name) {
+          Tscreen s = RCFindScreenName(n->name);
+          if (s != screen) {
+            w->UnMap();
+            w->Map(s);
           }
         }
         break;
@@ -563,13 +575,14 @@ static byte RCSteps(run *r) {
         break;
       case STDERR:
         argv = n->x.v.argv;
-        while (*argv)
-          printk("" SS " ", *argv++);
-        printk("\n");
+        while (*argv) {
+          log(INFO) << Chars::from_c(*argv++) << " ";
+        }
+        log(INFO) << "\n";
         break;
       case SYNTHETICKEY:
-        if (W)
-          SyntheticKey(W, n->x.f.a, n->x.f.flag, strlen(n->name), n->name);
+        if (w)
+          SyntheticKey(w, n->x.f.a, n->x.f.flag, strlen(n->name), n->name);
         break;
       case WAIT:
         /* remember the window name we are waiting for */
@@ -578,7 +591,7 @@ static byte RCSteps(run *r) {
         break;
       case WINDOW:
         if (n->name)
-          W = (widget)RCFindWindowName(n->name);
+          w = (Twidget)RCFindWindowName(n->name);
         else {
           ldat i = applyflagx(n);
           flag = n->x.f.plus_minus;
@@ -596,45 +609,48 @@ static byte RCSteps(run *r) {
 
           if (flag == 0) {
             if (i == 0) {
-              W = All->FirstScreen->FocusW;
-              if (W && !IS_WINDOW(W))
-                W = (widget)0;
+              w = All->Screens.First->FocusW();
+              if (w && !IS_WINDOW(w))
+                w = (Twidget)0;
             } else {
               if (i > 0)
                 i--;
-              W = ForwardWindow(All->FirstScreen->FirstW);
+              w = ForwardWindow(All->Screens.First->Widgets.First);
             }
           }
 
-          if (W) {
-            while (i > 0 && W) {
+          if (w) {
+            while (i > 0 && w) {
               i--;
-              if ((SkipW = ForwardWindow(W->Next)))
-                W = SkipW;
-              else {
-                S = ScreenOf(W);
-                W = NULL;
-                if (S)
-                  while ((S = S->Next) && !(W = ForwardWindow(S->FirstW)))
+              if ((SkipW = ForwardWindow(w->Next))) {
+                w = SkipW;
+              } else {
+                screen = ScreenOf(w);
+                w = NULL;
+                if (screen)
+                  while ((screen = screen->NextScreen()) &&
+                         !(w = ForwardWindow(screen->Widgets.First)))
                     ;
               }
             }
-            while (i < 0 && W) {
+            while (i < 0 && w) {
               i++;
-              if ((SkipW = BackwardWindow(W->Prev)))
-                W = SkipW;
-              else {
-                S = ScreenOf(W);
-                W = NULL;
-                if (S)
-                  while ((S = S->Prev) && !(W = BackwardWindow(S->LastW)))
-                    ;
+              if ((SkipW = BackwardWindow(w->Prev))) {
+                w = SkipW;
+              } else {
+                screen = ScreenOf(w);
+                w = NULL;
+                if (screen) {
+                  while ((screen = screen->PrevScreen()) &&
+                         !(w = BackwardWindow(screen->Widgets.Last))) {
+                  }
+                }
               }
             }
           }
         }
-        r->W = W ? W->Id : NOID;
-        S = ScreenOf(W);
+        r->W = w ? w->Id : NOID;
+        screen = ScreenOf(w);
         if (n->body) /* enter function */
           state = Sbody;
         break;
@@ -642,30 +658,36 @@ static byte RCSteps(run *r) {
         BeepHW();
         break;
       case CENTER:
-        if (W && IS_WINDOW(W))
-          CenterWindow((window)W);
+        if (w && IS_WINDOW(w))
+          CenterWindow((Twindow)w);
         break;
       case CLOSE:
-        if (W)
-          AskCloseWidget(W);
+        if (w)
+          AskCloseWidget(w);
+        break;
+      case COPY:
+        All->Clipboard->dup(All->Selection);
         break;
       case KILL:
         /* BRUTAL! */
-        if (W) {
-          msgport M = W->Owner;
+        if (w) {
+          Tmsgport M = w->Owner;
           /*
            * try not to exagerate with brutality:
            * allow deleting remote clients msgports only
            */
           if (M->RemoteData.FdSlot != NOSLOT) {
             Ext(Remote, KillSlot)(M->RemoteData.FdSlot);
-            W = NULL;
-            S = NULL;
+            w = NULL;
+            screen = NULL;
             r->W = NOID;
           } else
-            AskCloseWidget(W);
+            AskCloseWidget(w);
         }
         break;
+      case PASTE:
+        All->Clipboard->paste();
+        break;
       case QUIT:
         Quit(0);
         break;
@@ -676,56 +698,61 @@ static byte RCSteps(run *r) {
         ShowWinList(C);
         break;
       case FOCUS:
-        if (W && S) {
+        if (w && screen) {
           flag = n->x.f.flag;
           if (flag == FL_TOGGLE)
-            flag = (S->FocusW == (widget)W) ? FL_OFF : FL_ON;
+            flag = (screen->FocusW() == (Twidget)w) ? FL_OFF : FL_ON;
 
-          if (flag == FL_ON && S != All->FirstScreen)
-            S->Focus();
+          if (flag == FL_ON && screen != All->Screens.First)
+            screen->Focus();
 
           if (flag == FL_ON) {
-            W->Focus();
+            w->Focus();
           } else {
-            Do(Focus, window)(NULL);
+            Swindow::Focus(NULL);
           }
         }
         break;
       case MAXIMIZE:
       case FULLSCREEN:
-        if (W && IS_WINDOW(W) && S)
-          MaximizeWindow((window)W, n->id == FULLSCREEN);
+        if (w && IS_WINDOW(w) && screen)
+          MaximizeWindow((Twindow)w, n->id == FULLSCREEN);
         break;
       case LOWER:
-        if (W && S)
-          LowerWidget(W, tfalse);
+        if (w && screen)
+          LowerWidget(w, false);
         break;
       case RAISE:
-        if (W && S)
-          RaiseWidget(W, tfalse);
+        if (w && screen) {
+          RaiseWidget(w, false);
+        }
         break;
       case RAISELOWER:
-        if (W && S) {
-          if ((widget)W == S->FirstW)
-            LowerWidget(W, ttrue);
-          else
-            RaiseWidget(W, ttrue);
+        if (w && screen) {
+          if ((Twidget)w == screen->Widgets.First) {
+            LowerWidget(w, true);
+          } else {
+            RaiseWidget(w, true);
+          }
         }
         break;
       case ROLL:
-        if (W && IS_WINDOW(W)) {
+        if (w && IS_WINDOW(w)) {
           flag = n->x.f.flag;
-          if (flag == FL_TOGGLE)
-            flag = W->Attr & WINDOW_ROLLED_UP ? FL_OFF : FL_ON;
-          RollUpWindow((window)W, flag == FL_ON);
+          if (flag == FL_TOGGLE) {
+            flag = w->Attr & WINDOW_ROLLED_UP ? FL_OFF : FL_ON;
+          }
+          RollUpWindow((Twindow)w, flag == FL_ON);
         }
         break;
       case USERFUNC:
-        if (n->x.func || ((f = LookupNodeName(n->name, FuncList)) && (n->x.func = f->body)))
+        if (n->x.func || ((f = LookupNodeName(n->name, FuncList)) && (n->x.func = f->body))) {
           /* stored into n->f.func to speedup future references */
           state = Sfunc;
-        else /* user function not found. abort. */
+        } else {
+          /* user function not found. abort. */
           state = Serr;
+        }
         break;
       default:
         /* unknown command. abort. */
@@ -737,18 +764,20 @@ static byte RCSteps(run *r) {
       /* next instruction */
       if (!n || !(n = n->next)) {
         /* end of function. pop the stack. */
-        if (r->depth)
+        if (r->depth) {
           n = r->stack[--r->depth];
-        else /* stack empty. exit. */
-          break;
+        } else {
+          break; /* stack empty. exit. */
+        }
       }
     } else if (state == Sfunc || state == Sbody) {
       /* enter f or n->body */
       if (r->depth < MAX_RUNSTACK) {
         r->stack[r->depth++] = n->next;
         n = (state == Sfunc) ? n->x.func : n->body;
-      } else /* stack overflow */
-        break;
+      } else {
+        break; /* stack overflow */
+      }
     } else if (state == Ssleep || state == Swait || state == Sinter) {
       /* prepare to resume from next instruction after sleeping */
       r->stack[r->depth] = n->next;
@@ -825,29 +854,29 @@ static byte RCSleep(timevalue *_t) {
 /*
  * this is the real Initialization / Restart function.
  *
- * kill the queues, reload .twinrc and restart queues
+ * kill the queues, reload ~/.config/twin/twinrc and restart queues
  */
 static void RCReload(void) {
-  module M;
-  byte (*mod_rcload)(void) = NULL;
+  Tmodule M;
+  bool (*mod_rcload)(Tdisplay) = NULL;
   byte success;
 
-  if ((M = DlLoad(RCParseSo)))
+  if ((M = DlLoad(RCParseSo))) {
     mod_rcload = M->DoInit;
+  }
 #if 0
-    /* this would garble -hw=tty display */
-    else
-        printk("twin: failed to load the RC parser:\n"
-                "      " SS "\n", Errstr);
+  else { /* this would garble -hw=tty display */
+    log(ERROR) << "twin: failed to load the RC parser:\n      " << Errstr << "\n";
+  }
 #endif
 
-  success = mod_rcload && mod_rcload();
+  success = mod_rcload && mod_rcload(NULL);
 
   if (M)
     DlUnload(RCParseSo);
 
   if (success) {
-    QueuedDrawArea2FullScreen = ttrue;
+    QueuedDrawArea2FullScreen = true;
 
     ResetBorderPattern();
     RCKillAll();
@@ -882,14 +911,14 @@ static void RCWake(void) {
 }
 
 /* wake up queues when the wanted window appears */
-static void RCWake4Window(window W) {
-  str Name = W->Name;
+static void RCWake4Window(Twindow w) {
+  str Name = w->Name;
   run **p = &Wait, *r;
 
   if (Name)
     while ((r = *p)) {
       if (!strcmp(r->SW.Name, Name)) {
-        r->W = W->Id;
+        r->W = w->Id;
         RCRemove(p); /* p does not change but *p is now the next run */
         RCAddFirst(r, Run);
       } else
@@ -910,8 +939,8 @@ static byte MouseClickReleaseSameCtx(uldat W1, uldat W2, ldat clickCtx, ldat rel
 
 /* handle incoming messages */
 byte RC_VMQueue(const wm_ctx *C) {
-  uldat ClickWinId = All->FirstScreen->ClickWindow ? All->FirstScreen->ClickWindow->Id : NOID;
-  widget W;
+  uldat ClickWinId = All->Screens.First->ClickWindow ? All->Screens.First->ClickWindow->Id : NOID;
+  Twidget w;
   ldat ctx;
   node n;
   run *r;
@@ -957,7 +986,7 @@ byte RC_VMQueue(const wm_ctx *C) {
     }
     break;
   case msg_map:
-    used = ttrue, RCWake4Window((window)C->W);
+    used = ttrue, RCWake4Window((Twindow)C->W);
     break;
   case msg_control:
     if (C->Code == MSG_CONTROL_OPEN) {
@@ -965,7 +994,7 @@ byte RC_VMQueue(const wm_ctx *C) {
       if (All->State != state_default)
         /*
          * return to state_default, and rely on RCReload()
-         * to set QueuedDrawArea2FullScreen = ttrue
+         * to set QueuedDrawArea2FullScreen = true
          */
         ForceRelease(C);
 
@@ -978,9 +1007,9 @@ byte RC_VMQueue(const wm_ctx *C) {
       if (n && (r = RCNew(n))) {
         used = ttrue;
         CopyMem(C, &r->C, sizeof(wm_ctx));
-        W = All->FirstScreen->FocusW;
-        r->W = W ? W->Id : NOID;
-        r->C.ByMouse = tfalse;
+        w = All->Screens.First->FocusW();
+        r->W = w ? w->Id : NOID;
+        r->C.ByMouse = false;
         /* to preserve execution orded, run it right now ! */
         RCRun();
       }
@@ -1007,64 +1036,66 @@ void QuitRC(void) {
   shm_quit();
 }
 
-#define COD_COMMON_FIRST COD_COMMON_DRAG
+enum {
+  COD_COMMON_DRAG = COD_RESERVED,
+  COD_COMMON_RESIZE = COD_RESERVED + 1,
+  COD_COMMON_SCROLL = COD_RESERVED + 2,
+  COD_COMMON_CENTER = COD_RESERVED + 3,
+  COD_COMMON_MAXIMIZE = COD_RESERVED + 4,
+  COD_COMMON_FULLSCREEN = COD_RESERVED + 5,
+  COD_COMMON_ROLLTOGGLE = COD_RESERVED + 6,
+  COD_COMMON_RAISELOWER = COD_RESERVED + 7,
+  COD_COMMON_UNFOCUS = COD_RESERVED + 8,
+  COD_COMMON_NEXT = COD_RESERVED + 9,
+  COD_COMMON_WINDOWLIST = COD_RESERVED + 10,
+  COD_COMMON_REFRESH = COD_RESERVED + 11,
+  COD_COMMON_HOTKEY = COD_RESERVED + 12,
+  COD_COMMON_CLOSE = COD_RESERVED + 13,
 
-#define COD_COMMON_DRAG (COD_RESERVED)
-#define COD_COMMON_RESIZE (COD_RESERVED + 1)
-#define COD_COMMON_SCROLL (COD_RESERVED + 2)
-#define COD_COMMON_CENTER (COD_RESERVED + 3)
-#define COD_COMMON_MAXIMIZE (COD_RESERVED + 4)
-#define COD_COMMON_FULLSCREEN (COD_RESERVED + 5)
-#define COD_COMMON_ROLLTOGGLE (COD_RESERVED + 6)
-#define COD_COMMON_RAISELOWER (COD_RESERVED + 7)
-#define COD_COMMON_UNFOCUS (COD_RESERVED + 8)
-#define COD_COMMON_NEXT (COD_RESERVED + 9)
-#define COD_COMMON_WINDOWLIST (COD_RESERVED + 10)
-#define COD_COMMON_REFRESH (COD_RESERVED + 11)
-#define COD_COMMON_HOTKEY (COD_RESERVED + 12)
-#define COD_COMMON_CLOSE (COD_RESERVED + 13)
-
-#define COD_COMMON_LAST COD_COMMON_CLOSE
+  COD_COMMON_FIRST = COD_COMMON_DRAG,
+  COD_COMMON_LAST = COD_COMMON_CLOSE,
+};
 
 static byte USEDefaultCommonMenu(void) {
-  menu Menu;
-  menuitem Item;
-  window W;
-  row Row;
+  Tmenu Menu;
+  Tmenuitem item;
+  Twindow w;
+  Trow Row;
 
-  if (!(Menu = New(menu)(Ext(WM, MsgPort), (tcolor)0, (tcolor)0, (tcolor)0, (tcolor)0, (tcolor)0,
-                         (tcolor)0, ttrue)))
+  if (!(Menu = Smenu::Create(Ext(WM, MsgPort), (tcolor)0, (tcolor)0, (tcolor)0, (tcolor)0,
+                             (tcolor)0, (tcolor)0, ttrue)))
     return tfalse;
 
-  if ((W = Win4Menu(Menu)) && (Item = Item4Menu(Menu, W, ttrue, 8, " Window ")) &&
+  if ((w = Swindow::Create4Menu(Menu)) &&
+      (item = Smenuitem::Create4Menu(Menu, w, 0, ttrue, 8, " Window ")) &&
 
       /* we cannot create rows with codes >= COD_RESERVED... */
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Move        ")) && (Row->Code = COD_COMMON_DRAG) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Resize      ")) && (Row->Code = COD_COMMON_RESIZE) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Scroll      ")) && (Row->Code = COD_COMMON_SCROLL) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Center      ")) && (Row->Code = COD_COMMON_CENTER) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Maximize    ")) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Move        ")) && (Row->Code = COD_COMMON_DRAG) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Resize      ")) && (Row->Code = COD_COMMON_RESIZE) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Scroll      ")) && (Row->Code = COD_COMMON_SCROLL) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Center      ")) && (Row->Code = COD_COMMON_CENTER) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Maximize    ")) &&
       (Row->Code = COD_COMMON_MAXIMIZE) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Full Screen ")) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Full screen ")) &&
       (Row->Code = COD_COMMON_FULLSCREEN) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Roll Up     ")) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Roll Up     ")) &&
       (Row->Code = COD_COMMON_ROLLTOGGLE) &&
-      Row4Menu(W, 0, ROW_IGNORE, 13, "\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4") &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Raise/Lower ")) &&
+      Row4Menu(w, 0, ROW_IGNORE, 13, "\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4") &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Raise/Lower ")) &&
       (Row->Code = COD_COMMON_RAISELOWER) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " UnFocus     ")) && (Row->Code = COD_COMMON_UNFOCUS) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Next        ")) && (Row->Code = COD_COMMON_NEXT) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " List...     ")) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " UnFocus     ")) && (Row->Code = COD_COMMON_UNFOCUS) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Next        ")) && (Row->Code = COD_COMMON_NEXT) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " List...     ")) &&
       (Row->Code = COD_COMMON_WINDOWLIST) &&
-      Row4Menu(W, 0, ROW_IGNORE, 13, "\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4") &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Refresh     ")) && (Row->Code = COD_COMMON_REFRESH) &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Send HotKey ")) && (Row->Code = COD_COMMON_HOTKEY) &&
-      Row4Menu(W, 0, ROW_IGNORE, 13, "\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4") &&
-      (Row = Row4Menu(W, 0, ROW_ACTIVE, 13, " Close       ")) && (Row->Code = COD_COMMON_CLOSE)) {
+      Row4Menu(w, 0, ROW_IGNORE, 13, "\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4") &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Refresh     ")) && (Row->Code = COD_COMMON_REFRESH) &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Send HotKey ")) && (Row->Code = COD_COMMON_HOTKEY) &&
+      Row4Menu(w, 0, ROW_IGNORE, 13, "\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4\xC4") &&
+      (Row = Row4Menu(w, 0, ROW_ACTIVE, 13, " Close       ")) && (Row->Code = COD_COMMON_CLOSE)) {
 
     /* success */
 
-    Item->Left = 0; /* remove padding */
+    item->Left = 0; /* remove padding */
 
     if (All->CommonMenu) {
       All->CommonMenu->Delete();
@@ -1086,7 +1117,7 @@ byte InitRC(void) {
        NULL,
        NULL,
        NULL,
-       NULL,
+       {NULL},
        {
            {
                0,
@@ -1097,7 +1128,7 @@ byte InitRC(void) {
        NULL,
        NULL,
        NULL,
-       NULL,
+       {NULL},
        {
            {
                0,
@@ -1108,7 +1139,7 @@ byte InitRC(void) {
        NULL,
        NULL,
        NULL,
-       NULL,
+       {NULL},
        {
            {
                0,
@@ -1128,7 +1159,7 @@ byte InitRC(void) {
        NULL,
        NULL,
        NULL,
-       NULL,
+       {NULL},
        {
            {
                0,
@@ -1142,7 +1173,7 @@ byte InitRC(void) {
        NULL,
        NULL,
        NULL,
-       NULL,
+       {NULL},
        {
            {
                0,
@@ -1153,7 +1184,7 @@ byte InitRC(void) {
        NULL,
        NULL,
        NULL,
-       NULL,
+       {NULL},
        {
            {
                '+',
@@ -1189,7 +1220,7 @@ byte InitRC(void) {
                                NULL,
                                N + 8,
                                NULL,
-                               NULL,
+                               {NULL},
                                {{
                                    0,
                                    0,
@@ -1207,7 +1238,7 @@ byte InitRC(void) {
                                NULL,
                                NULL,
                                NULL,
-                               NULL,
+                               {NULL},
                                {
                                    {
                                        0,
@@ -1221,7 +1252,7 @@ byte InitRC(void) {
                                "0",
                                M + 1,
                                N + 13,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_0,
                                }}},
@@ -1230,7 +1261,7 @@ byte InitRC(void) {
                                "1",
                                M + 2,
                                N + 7,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_1,
                                }}},
@@ -1239,7 +1270,7 @@ byte InitRC(void) {
                                "2",
                                M + 3,
                                N + 6,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_2,
                                }}},
@@ -1248,7 +1279,7 @@ byte InitRC(void) {
                                "TS",
                                M + 4,
                                N + 0,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_TITLE | CTX_SIDE,
                                }}},
@@ -1257,7 +1288,7 @@ byte InitRC(void) {
                                "C",
                                M + 5,
                                N + 1,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_CORNER,
                                }}},
@@ -1266,7 +1297,7 @@ byte InitRC(void) {
                                "B",
                                M + 6,
                                N + 2,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_BARS,
                                }}},
@@ -1275,7 +1306,7 @@ byte InitRC(void) {
                                "R",
                                M + 8,
                                M + 7,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_ROOT,
                                }}},
@@ -1288,7 +1319,7 @@ byte InitRC(void) {
                                "R",
                                M + 9,
                                N + 10,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_ROOT,
                                }}},
@@ -1297,16 +1328,16 @@ byte InitRC(void) {
                                "A",
                                M + 10,
                                K + 1,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_ANY,
                                }}},
-                              /* Mouse  H1  M  Interactive Screen */
+                              /* Mouse  H1  M  Interactive screen */
                               {HOLD_LEFT | PRESS_,
                                "M",
                                NULL,
                                M + 11,
-                               NULL,
+                               {NULL},
                                {{
                                    CTX_MENU,
                                }}},
@@ -1314,7 +1345,7 @@ byte InitRC(void) {
                                NULL,
                                NULL,
                                NULL,
-                               NULL,
+                               {NULL},
                                {
                                    {
                                        0,
@@ -1363,7 +1394,7 @@ byte InitRC(void) {
     UpdateOptionWin();
     FillButtonWin();
     HideMenu(!!(All->SetUp->Flags & setup_menu_hide));
-    All->FirstScreen->DrawMenu(0, TW_MAXDAT);
+    All->Screens.First->DrawMenu(0, TW_MAXDAT);
 
     return ttrue;
   }
