@@ -9,7 +9,10 @@
 #ifndef TWIN_RCRUN_H
 #define TWIN_RCRUN_H
 
-#define GLOBAL_MAX 7
+struct wm_ctx;
+
+enum { GLOBAL_MAX = 7 };
+
 extern node Globals[GLOBAL_MAX];
 extern byte GlobalsAreStatic;
 
@@ -32,7 +35,7 @@ void QuitRC(void);
 
 node LookupNodeName(cstr name, node head);
 
-trune *RCFindBorderPattern(window W, byte Border);
+trune *RCFindBorderPattern(Twindow w, byte border);
 
 byte RC_VM(timevalue *t);
 byte RC_VMQueue(const wm_ctx *C);
