@@ -10,21 +10,21 @@
 #define TWIN_REMOTE_H
 
 typedef void (*handler_io)(int Fd, uldat Slot);
-typedef void (*handler_obj)(int Fd, obj HandlerObj);
-typedef void (*handler_window)(int Fd, window Window);
+typedef void (*handler_obj)(int Fd, Tobj HandlerObj);
+typedef void (*handler_window)(int Fd, Twindow Window);
 
 uldat RegisterRemoteFd(int Fd, handler_io HandlerIO);
-uldat RegisterRemote(int Fd, obj HandlerData, handler_obj HandlerObj);
+uldat RegisterRemote(int Fd, Tobj HandlerData, handler_obj HandlerObj);
 void UnRegisterRemote(uldat Slot);
-byte RegisterWindowFdIO(window Window, handler_window HandlerWindow);
-void UnRegisterWindowFdIO(window Window);
+byte RegisterWindowFdIO(Twindow Window, handler_window HandlerWindow);
+void UnRegisterWindowFdIO(Twindow Window);
 uldat RemoteWriteQueue(uldat Slot, uldat len, const void *data);
 #define RemoteWindowWriteQueue(Window, len, data)                                                  \
   RemoteWriteQueue((Window)->RemoteData.FdSlot, (len), (data))
 byte RemoteFlush(uldat Slot);
 #define RemoteWindowFlush(Window) RemoteFlush((Window)->RemoteData.FdSlot)
 
-msgport RemoteGetMsgPort(uldat Slot);
+Tmsgport RemoteGetMsgPort(uldat Slot);
 
 void RemoteFlushAll(void);
 void RemoteEvent(int FdNum, fd_set *FdSet);
@@ -45,8 +45,8 @@ void RemoteCouldWrite(uldat Slot);
 
 extern uldat FdWQueued;
 
-void RegisterMsgPort(msgport MsgPort, uldat Slot);
-void UnRegisterMsgPort(msgport MsgPort);
+void RegisterMsgPort(Tmsgport MsgPort, uldat Slot);
+void UnRegisterMsgPort(Tmsgport MsgPort);
 
 void remoteKillSlot(uldat slot);
 
