@@ -7,38 +7,25 @@
  *
  */
 
-/* FIXME: remove these */
-#define TWS_ggroup_Id TWS_group_Id
-#define TWS_ggroup_Prev TWS_group_Prev
-#define TWS_ggroup_Next TWS_group_Next
-#define TWS_ggroup_Parent TWS_group_Parent
-#define TWS_ggroup_Prev_List TWS_group_Prev_List
-#define TWS_ggroup_Next_List TWS_group_Next_List
-#define TWS_ggroup_Parent_List TWS_group_Parent_List
-#define TWS_ggroup_Owner TWS_group_Owner
-#define TWS_ggroup_MsgPort TWS_group_MsgPort
-#define TWS_ggroup_FirstG TWS_group_FirstG
-#define TWS_ggroup_LastG TWS_group_LastG
-#define TWS_ggroup_SelectG TWS_group_SelectG
-#define TWS_ggroup_ChildrenG_List TWS_group_ChildrenG_List
-
-#define TWScase(objtype, field, fieldtype)                                                         \
-  case CAT4(TWS_, objtype, _, field):                                                              \
+#define TWScase4(objtype, fieldname, field, fieldtype)                                             \
+  case CAT4(TWS_, objtype, _, fieldname):                                                          \
     /* ensure type size WAS negotiated */                                                          \
     if (CAT(TWS_, fieldtype) <= TWS_tcolor || CAT(TWS_, fieldtype) >= TWS_highest ||               \
-        AlienSizeof(fieldtype, Slot)) {                                                            \
-      TSF->CAT(TWS_field_, fieldtype) = (fieldtype)x->field;                                       \
+        AlienSizeof(fieldtype, ctx.Slot)) {                                                        \
+      TSF->CAT(TWS_field_, fieldtype) = x->field;                                                  \
       TSF->type = CAT(TWS_, fieldtype);                                                            \
       break;                                                                                       \
     }                                                                                              \
     return tfalse
 
-#define TWScaseUSE(objtype, use, field, fieldtype)                                                 \
-  case CAT6(TWS_, objtype, _USE_, use, _, field):                                                  \
+#define TWScase(objtype, field, fieldtype) TWScase4(objtype, field, field, fieldtype)
+
+#define TWScaseUSE(objtype, use, fieldname, field, fieldtype)                                      \
+  case CAT6(TWS_, objtype, _USE_, use, _, fieldname):                                              \
     /* ensure type size WAS negotiated */                                                          \
     if (CAT(TWS_, fieldtype) <= TWS_tcolor || CAT(TWS_, fieldtype) >= TWS_highest ||               \
-        AlienSizeof(fieldtype, Slot)) {                                                            \
-      TSF->CAT(TWS_field_, fieldtype) = (fieldtype)x->USE.use.field;                               \
+        AlienSizeof(fieldtype, ctx.Slot)) {                                                        \
+      TSF->CAT(TWS_field_, fieldtype) = x->USE.use.field;                                          \
       TSF->type = CAT(TWS_, fieldtype);                                                            \
       break;                                                                                       \
     }                                                                                              \
@@ -48,8 +35,8 @@
   case CAT5(TWS_, objtype, _, field, V, n):                                                        \
     /* ensure type size WAS negotiated */                                                          \
     if (CAT(TWS_, fieldtype) <= TWS_tcolor || CAT(TWS_, fieldtype) >= TWS_highest ||               \
-        AlienSizeof(fieldtype, Slot)) {                                                            \
-      TSF->CAT(TWS_field_, fieldtype) = (fieldtype)x->field[n];                                    \
+        AlienSizeof(fieldtype, ctx.Slot)) {                                                        \
+      TSF->CAT(TWS_field_, fieldtype) = x->field[n];                                               \
       TSF->type = CAT(TWS_, fieldtype);                                                            \
       break;                                                                                       \
     }                                                                                              \
@@ -59,8 +46,8 @@
   case CAT8(TWS_, objtype, _USE_, use, _, field, V, n):                                            \
     /* ensure type size WAS negotiated */                                                          \
     if (CAT(TWS_, fieldtype) <= TWS_tcolor || CAT(TWS_, fieldtype) >= TWS_highest ||               \
-        AlienSizeof(fieldtype, Slot)) {                                                            \
-      TSF->CAT(TWS_field_, fieldtype) = (fieldtype)x->USE.use.field[n];                            \
+        AlienSizeof(fieldtype, ctx.Slot)) {                                                        \
+      TSF->CAT(TWS_field_, fieldtype) = x->USE.use.field[n];                                       \
       TSF->type = CAT(TWS_, fieldtype);                                                            \
       break;                                                                                       \
     }                                                                                              \
@@ -94,34 +81,34 @@
     TSF->type = TWS_vec | CAT(TWS_, _type);                                                        \
     break
 
-#define fieldDelta(field) ((udat)(size_t) & (((obj_entry)0)->field))
-#define fieldTypeDelta(type, field) ((udat)(size_t) & (((type)0)->field))
+#define fieldDelta(field) ((udat)(size_t)&(((TobjEntry)0)->field))
+#define OFFSET_OF(type, field) ((udat)(size_t)&(((type)0)->field))
 
-#define sockAllocListPrevObjs(F, len) sockAllocListDeltaObjs((obj)(F), (len), fieldDelta(Prev))
-#define sockAllocListNextObjs(F, len) sockAllocListDeltaObjs((obj)(F), (len), fieldDelta(Next))
-#define sockAllocListParentObjs(F, len) sockAllocListDeltaObjs((obj)(F), (len), fieldDelta(Parent))
+#define sockAllocListPrevObjs(F, len) sockAllocListDeltaObjs((F), (len), fieldDelta(Prev))
+#define sockAllocListNextObjs(F, len) sockAllocListDeltaObjs((F), (len), fieldDelta(Next))
+#define sockAllocListParentObjs(F, len) sockAllocListDeltaObjs((F), (len), fieldDelta(Parent))
 
-static tobj *sockAllocListDeltaObjs(obj F, topaque *len, udat fdelta) {
+static tobj *sockAllocListDeltaObjs(Tobj F, topaque *len, udat fdelta) {
   topaque L = 0;
   tobj *_LW, *LW;
-  obj O = F;
+  Tobj O = F;
   while (O) {
     L++;
-    O = *(obj *)(fdelta + (byte *)O);
+    O = deserialize<Tobj>(O, fdelta);
   }
   if ((_LW = LW = (tobj *)AllocMem(L *= sizeof(tobj)))) {
     *len = L;
     O = F;
     while (O) {
       *LW++ = O->Id;
-      O = *(obj *)(fdelta + (byte *)O);
+      O = deserialize<Tobj>(O, fdelta);
     }
   }
   return _LW;
 }
 
-static byte sockStatObj(obj o, tsfield TSF) {
-  obj_entry x = (obj_entry)o;
+static byte sockStatObj(SockCtx &ctx, Tobj o, tsfield TSF) {
+  TobjEntry x = (TobjEntry)o;
   switch (TSF->label) {
   case TWS_obj_Id:
     break;
@@ -149,15 +136,15 @@ static byte sockStatObj(obj o, tsfield TSF) {
   default:
     return tfalse;
   }
-  TSF->TWS_field_obj = (obj)x;
+  TSF->TWS_field_obj = x;
   TSF->type = TWS_obj;
   return ttrue;
 }
 
-static byte sockStatWidget(widget x, tsfield TSF) {
+static byte sockStatWidget(SockCtx &ctx, Twidget x, tsfield TSF) {
   switch (TSF->label) {
-    TWScase(widget, FirstW, obj);
-    TWScase(widget, LastW, obj);
+    TWScase4(widget, FirstW, Widgets.First, obj);
+    TWScase4(widget, LastW, Widgets.Last, obj);
     TWScase(widget, SelectW, obj);
     TWScase(widget, Left, dat);
     TWScase(widget, Up, dat);
@@ -172,23 +159,23 @@ static byte sockStatWidget(widget x, tsfield TSF) {
     TWScase(widget, Owner, obj);
     TWScase(widget, USE_Fill, tcell);
   case TWS_widget_ChildrenW_List:
-    TSF->TWS_field_vecV = sockAllocListNextObjs((obj)x->FirstW, &TSF->TWS_field_vecL);
+    TSF->TWS_field_vecV = sockAllocListNextObjs(x->Widgets.First, &TSF->TWS_field_vecL);
     TSF->type = TWS_vec | TWS_tobj;
     break;
   case TWS_widget_O_Prev_List:
-    TSF->TWS_field_vecV = sockAllocListDeltaObjs((obj)x->O_Prev, &TSF->TWS_field_vecL,
-                                                 fieldTypeDelta(widget, O_Prev));
+    TSF->TWS_field_vecV =
+        sockAllocListDeltaObjs(x->O_Prev, &TSF->TWS_field_vecL, OFFSET_OF(Twidget, O_Prev));
     TSF->type = TWS_vec | TWS_tobj;
     break;
   case TWS_widget_O_Next_List:
-    TSF->TWS_field_vecV = sockAllocListDeltaObjs((obj)x->O_Next, &TSF->TWS_field_vecL,
-                                                 fieldTypeDelta(widget, O_Next));
+    TSF->TWS_field_vecV =
+        sockAllocListDeltaObjs(x->O_Next, &TSF->TWS_field_vecL, OFFSET_OF(Twidget, O_Next));
     TSF->type = TWS_vec | TWS_tobj;
     break;
   default:
     return tfalse;
   }
-  /* correct for screen scrolling */
+  /* correct for Tscreen scrolling */
   if (x->Parent && IS_SCREEN(x->Parent)) {
     if (TSF->label == TWS_widget_Left)
       TSF->TWS_field_scalar -= x->Parent->XLogic;
@@ -198,7 +185,7 @@ static byte sockStatWidget(widget x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatGadget(gadget x, tsfield TSF) {
+static byte sockStatGadget(SockCtx &ctx, Tgadget x, tsfield TSF) {
   switch (TSF->label) {
     TWScase(gadget, ColText, tcolor);
     TWScase(gadget, ColSelect, tcolor);
@@ -210,17 +197,17 @@ static byte sockStatGadget(gadget x, tsfield TSF) {
     TWScase(gadget, G_Next, obj);
     TWScase(gadget, Group, obj);
   case TWS_gadget_G_Prev_List:
-    TSF->TWS_field_vecV = sockAllocListDeltaObjs((obj)x->G_Prev, &TSF->TWS_field_vecL,
-                                                 fieldTypeDelta(gadget, G_Prev));
+    TSF->TWS_field_vecV =
+        sockAllocListDeltaObjs(x->G_Prev, &TSF->TWS_field_vecL, OFFSET_OF(Tgadget, G_Prev));
     TSF->type = TWS_vec | TWS_tobj;
     break;
   case TWS_gadget_G_Next_List:
-    TSF->TWS_field_vecV = sockAllocListDeltaObjs((obj)x->G_Next, &TSF->TWS_field_vecL,
-                                                 fieldTypeDelta(gadget, G_Next));
+    TSF->TWS_field_vecV =
+        sockAllocListDeltaObjs(x->G_Next, &TSF->TWS_field_vecL, OFFSET_OF(Tgadget, G_Next));
     TSF->type = TWS_vec | TWS_tobj;
     break;
   default:
-    if (G_USE((gadget)x, USETEXT)) {
+    if (G_USE((Tgadget)x, USETEXT)) {
       switch (TSF->label) {
         TWScaseAvecUSE(gadget, T, Text, 0, trune, x->XWidth * x->YWidth);
         TWScaseAvecUSE(gadget, T, Text, 1, trune, x->XWidth * x->YWidth);
@@ -240,13 +227,13 @@ static byte sockStatGadget(gadget x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatWindow(window x, tsfield TSF) {
+static byte sockStatWindow(SockCtx &ctx, Twindow x, tsfield TSF) {
   switch (TSF->label) {
   case TWS_widget_Left:
   case TWS_widget_Up:
   case TWS_widget_XWidth:
   case TWS_widget_YWidth:
-    sockStatWidget((widget)x, TSF);
+    sockStatWidget(ctx, (Twidget)x, TSF);
     break;
 
     TWScase(window, Menu, obj);
@@ -281,19 +268,19 @@ static byte sockStatWindow(window x, tsfield TSF) {
     TWScase(window, WLogic, ldat);
     TWScase(window, HLogic, ldat);
   default:
-    if (W_USE((window)x, USECONTENTS)) {
+    if (W_USE((Twindow)x, USECONTENTS)) {
       switch (TSF->label) {
         TWScasevecUSE(window, C, Contents, tcell, x->WLogic * x->HLogic);
-        TWScaseUSE(window, C, HSplit, ldat);
+        TWScaseUSE(window, C, HSplit, HSplit, ldat);
       default:
         return tfalse;
       }
-    } else if (W_USE((window)x, USEROWS)) {
+    } else if (W_USE((Twindow)x, USEROWS)) {
       switch (TSF->label) {
-        TWScaseUSE(window, R, FirstRow, obj);
-        TWScaseUSE(window, R, LastRow, obj);
+        TWScaseUSE(window, R, FirstRow, Rows.First, obj);
+        TWScaseUSE(window, R, LastRow, Rows.Last, obj);
       case TWS_window_USE_R_ChildrenRow_List:
-        TSF->TWS_field_vecV = sockAllocListNextObjs((obj)x->USE.R.FirstRow, &TSF->TWS_field_vecL);
+        TSF->TWS_field_vecV = sockAllocListNextObjs(x->USE.R.Rows.First, &TSF->TWS_field_vecL);
         TSF->type = TWS_vec | TWS_tobj;
         break;
       default:
@@ -313,15 +300,15 @@ static byte sockStatWindow(window x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatScreen(screen x, tsfield TSF) {
+static byte sockStatScreen(SockCtx &ctx, Tscreen x, tsfield TSF) {
   switch (TSF->label) {
     TWScase(screen, NameLen, dat);
     TWScasevec(screen, Name, byte, x->NameLen);
   default:
-    if (S_USE((gadget)x, USEBG)) {
+    if (S_USE((Tgadget)x, USEBG)) {
       switch (TSF->label) {
-        TWScaseUSE(screen, B, BgWidth, dat);
-        TWScaseUSE(screen, B, BgHeight, dat);
+        TWScaseUSE(screen, B, BgWidth, BgWidth, dat);
+        TWScaseUSE(screen, B, BgHeight, BgHeight, dat);
         TWScasevecUSE(screen, B, Bg, tcell, x->USE.B.BgWidth * x->USE.B.BgHeight);
       default:
         return tfalse;
@@ -333,14 +320,14 @@ static byte sockStatScreen(screen x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatGroup(ggroup x, tsfield TSF) {
+static byte sockStatGroup(SockCtx &ctx, Tgroup x, tsfield TSF) {
   switch (TSF->label) {
-    TWScase(ggroup, FirstG, obj);
-    TWScase(ggroup, LastG, obj);
-    TWScase(ggroup, SelectG, obj);
+    TWScase4(group, FirstG, Gadgets.First, obj);
+    TWScase4(group, LastG, Gadgets.Last, obj);
+    TWScase(group, SelectG, obj);
   case TWS_group_ChildrenG_List:
-    TSF->TWS_field_vecV = sockAllocListDeltaObjs((obj)x->FirstG, &TSF->TWS_field_vecL,
-                                                 fieldTypeDelta(gadget, G_Next));
+    TSF->TWS_field_vecV =
+        sockAllocListDeltaObjs(x->Gadgets.First, &TSF->TWS_field_vecL, OFFSET_OF(Tgadget, G_Next));
     TSF->type = TWS_vec | TWS_tobj;
     break;
   default:
@@ -349,7 +336,7 @@ static byte sockStatGroup(ggroup x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatRow(row x, tsfield TSF) {
+static byte sockStatRow(SockCtx &ctx, Trow x, tsfield TSF) {
   switch (TSF->label) {
     TWScase(row, Code, udat);
     TWScase(row, Flags, byte);
@@ -362,7 +349,7 @@ static byte sockStatRow(row x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatMenuItem(menuitem x, tsfield TSF) {
+static byte sockStatMenuItem(SockCtx &ctx, Tmenuitem x, tsfield TSF) {
   switch (TSF->label) {
     /* missing: */
 #if 0
@@ -379,7 +366,7 @@ static byte sockStatMenuItem(menuitem x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatMenu(menu x, tsfield TSF) {
+static byte sockStatMenu(SockCtx &ctx, Tmenu x, tsfield TSF) {
   switch (TSF->label) {
     TWScase(menu, ColItem, tcolor);
     TWScase(menu, ColSelect, tcolor);
@@ -389,11 +376,11 @@ static byte sockStatMenu(menu x, tsfield TSF) {
     TWScase(menu, ColSelShtCut, tcolor);
     TWScase(menu, CommonItems, byte);
     TWScase(menu, FlagDefColInfo, byte);
-    TWScase(menu, FirstI, obj);
-    TWScase(menu, LastI, obj);
+    TWScase4(menu, FirstI, Items.First, obj);
+    TWScase4(menu, LastI, Items.Last, obj);
     TWScase(menu, SelectI, obj);
   case TWS_menu_ChildrenI_List:
-    TSF->TWS_field_vecV = sockAllocListNextObjs((obj)x->FirstI, &TSF->TWS_field_vecL);
+    TSF->TWS_field_vecV = sockAllocListNextObjs(x->Items.First, &TSF->TWS_field_vecL);
     TSF->type = TWS_vec | TWS_tobj;
     break;
   default:
@@ -402,35 +389,35 @@ static byte sockStatMenu(menu x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatMsgPort(msgport x, tsfield TSF) {
+static byte sockStatMsgPort(SockCtx &ctx, Tmsgport x, tsfield TSF) {
   switch (TSF->label) {
     TWScase(msgport, WakeUp, byte);
     TWScase(msgport, NameLen, byte);
     TWScasevec(msgport, Name, byte, x->NameLen);
-    TWScase(msgport, FirstMenu, obj);
-    TWScase(msgport, LastMenu, obj);
-    TWScase(msgport, FirstW, obj);
-    TWScase(msgport, LastW, obj);
-    TWScase(msgport, FirstGroup, obj);
-    TWScase(msgport, LastGroup, obj);
-    TWScase(msgport, FirstMutex, obj);
-    TWScase(msgport, LastMutex, obj);
+    TWScase4(msgport, FirstMenu, Menus.First, obj);
+    TWScase4(msgport, LastMenu, Menus.Last, obj);
+    TWScase4(msgport, FirstW, Widgets.First, obj);
+    TWScase4(msgport, LastW, Widgets.Last, obj);
+    TWScase4(msgport, FirstGroup, Groups.First, obj);
+    TWScase4(msgport, LastGroup, Groups.Last, obj);
+    TWScase4(msgport, FirstMutex, Mutexes.First, obj);
+    TWScase4(msgport, LastMutex, Mutexes.Last, obj);
   case TWS_msgport_ChildrenMenu_List:
-    TSF->TWS_field_vecV = sockAllocListNextObjs((obj)x->FirstMenu, &TSF->TWS_field_vecL);
+    TSF->TWS_field_vecV = sockAllocListNextObjs(x->Menus.First, &TSF->TWS_field_vecL);
     TSF->type = TWS_vec | TWS_tobj;
     break;
   case TWS_msgport_ChildrenW_List:
-    TSF->TWS_field_vecV = sockAllocListDeltaObjs((obj)x->FirstW, &TSF->TWS_field_vecL,
-                                                 fieldTypeDelta(widget, O_Next));
+    TSF->TWS_field_vecV =
+        sockAllocListDeltaObjs(x->Widgets.First, &TSF->TWS_field_vecL, OFFSET_OF(Twidget, O_Next));
     TSF->type = TWS_vec | TWS_tobj;
     break;
   case TWS_msgport_ChildrenGroup_List:
-    TSF->TWS_field_vecV = sockAllocListNextObjs((obj)x->FirstGroup, &TSF->TWS_field_vecL);
+    TSF->TWS_field_vecV = sockAllocListNextObjs(x->Groups.First, &TSF->TWS_field_vecL);
     TSF->type = TWS_vec | TWS_tobj;
     break;
   case TWS_msgport_ChildrenMutex_List:
-    TSF->TWS_field_vecV = sockAllocListDeltaObjs((obj)x->FirstMutex, &TSF->TWS_field_vecL,
-                                                 fieldTypeDelta(mutex, O_Next));
+    TSF->TWS_field_vecV =
+        sockAllocListDeltaObjs(x->Mutexes.First, &TSF->TWS_field_vecL, OFFSET_OF(Tmutex, O_Next));
     TSF->type = TWS_vec | TWS_tobj;
     break;
   default:
@@ -439,7 +426,7 @@ static byte sockStatMsgPort(msgport x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatMutex(mutex x, tsfield TSF) {
+static byte sockStatMutex(SockCtx &ctx, Tmutex x, tsfield TSF) {
   switch (TSF->label) {
     TWScase(mutex, O_Prev, obj);
     TWScase(mutex, O_Next, obj);
@@ -449,12 +436,12 @@ static byte sockStatMutex(mutex x, tsfield TSF) {
     TWScasevec(mutex, Name, byte, x->NameLen);
   case TWS_mutex_O_Prev_List:
     TSF->TWS_field_vecV =
-        sockAllocListDeltaObjs((obj)x->O_Prev, &TSF->TWS_field_vecL, fieldTypeDelta(mutex, O_Prev));
+        sockAllocListDeltaObjs(x->O_Prev, &TSF->TWS_field_vecL, OFFSET_OF(Tmutex, O_Prev));
     TSF->type = TWS_vec | TWS_tobj;
     break;
   case TWS_mutex_O_Next_List:
     TSF->TWS_field_vecV =
-        sockAllocListDeltaObjs((obj)x->O_Next, &TSF->TWS_field_vecL, fieldTypeDelta(mutex, O_Next));
+        sockAllocListDeltaObjs(x->O_Next, &TSF->TWS_field_vecL, OFFSET_OF(Tmutex, O_Next));
     TSF->type = TWS_vec | TWS_tobj;
     break;
   default:
@@ -463,28 +450,28 @@ static byte sockStatMutex(mutex x, tsfield TSF) {
   return ttrue;
 }
 
-static byte sockStatAll(all x, tsfield TSF) {
+static byte sockStatAll(SockCtx &ctx, Tall x, tsfield TSF) {
   switch (TSF->label) {
-    TWScase(all, FirstScreen, obj);
-    TWScase(all, LastScreen, obj);
-    TWScase(all, FirstMsgPort, obj);
-    TWScase(all, LastMsgPort, obj);
-    TWScase(all, FirstMutex, obj);
-    TWScase(all, LastMutex, obj);
+    TWScase4(all, FirstScreen, Screens.First, obj);
+    TWScase4(all, LastScreen, Screens.Last, obj);
+    TWScase4(all, FirstMsgPort, MsgPorts.First, obj);
+    TWScase4(all, LastMsgPort, MsgPorts.Last, obj);
+    TWScase4(all, FirstMutex, Mutexes.First, obj);
+    TWScase4(all, LastMutex, Mutexes.Last, obj);
     TWScase(all, DisplayWidth, dat);
     TWScase(all, DisplayHeight, dat);
     TWScase(all, BuiltinMenu, obj);
     TWScase(all, CommonMenu, obj);
   case TWS_all_ChildrenScreen_List:
-    TSF->TWS_field_vecV = sockAllocListNextObjs((obj)x->FirstScreen, &TSF->TWS_field_vecL);
+    TSF->TWS_field_vecV = sockAllocListNextObjs(x->Screens.First, &TSF->TWS_field_vecL);
     TSF->type = TWS_vec | TWS_tobj;
     break;
   case TWS_all_ChildrenMsgPort_List:
-    TSF->TWS_field_vecV = sockAllocListNextObjs((obj)x->FirstMsgPort, &TSF->TWS_field_vecL);
+    TSF->TWS_field_vecV = sockAllocListNextObjs(x->MsgPorts.First, &TSF->TWS_field_vecL);
     TSF->type = TWS_vec | TWS_tobj;
     break;
   case TWS_all_ChildrenMutex_List:
-    TSF->TWS_field_vecV = sockAllocListNextObjs((obj)x->FirstMutex, &TSF->TWS_field_vecL);
+    TSF->TWS_field_vecV = sockAllocListNextObjs(x->Mutexes.First, &TSF->TWS_field_vecL);
     TSF->type = TWS_vec | TWS_tobj;
     break;
   default:
@@ -493,7 +480,7 @@ static byte sockStatAll(all x, tsfield TSF) {
   return ttrue;
 }
 
-static void sockStat(obj x, udat n, const byte *in) {
+static void sockStat(SockCtx &ctx, Tobj x, udat n, const byte *in) {
   udat i, j;
   tsfield TSF;
   uldat len, q;
@@ -504,40 +491,40 @@ static void sockStat(obj x, udat n, const byte *in) {
   if (ok) {
     for (i = j = 0; j < n; j++) {
       Pop(in, udat, TSF[i].label);
-      switch (x->Id >> magic_shift) {
-      case widget_magic_id:
-        ok = sockStatWidget((widget)x, TSF + i) || sockStatObj(x, TSF + i);
+      switch (x->Id >> class_byte_shift) {
+      case Twidget_class_byte:
+        ok = sockStatWidget(ctx, (Twidget)x, TSF + i) || sockStatObj(ctx, x, TSF + i);
         break;
-      case gadget_magic_id:
-        ok = sockStatGadget((gadget)x, TSF + i) || sockStatWidget((widget)x, TSF + i) ||
-             sockStatObj(x, TSF + i);
+      case Tgadget_class_byte:
+        ok = sockStatGadget(ctx, (Tgadget)x, TSF + i) || sockStatWidget(ctx, (Twidget)x, TSF + i) ||
+             sockStatObj(ctx, x, TSF + i);
         break;
-      case window_magic_id:
-        ok = sockStatWindow((window)x, TSF + i) || sockStatWidget((widget)x, TSF + i) ||
-             sockStatObj(x, TSF + i);
+      case Twindow_class_byte:
+        ok = sockStatWindow(ctx, (Twindow)x, TSF + i) || sockStatWidget(ctx, (Twidget)x, TSF + i) ||
+             sockStatObj(ctx, x, TSF + i);
         break;
-      case screen_magic_id:
-        ok = sockStatScreen((screen)x, TSF + i) || sockStatWidget((widget)x, TSF + i) ||
-             sockStatObj(x, TSF + i);
+      case Tscreen_class_byte:
+        ok = sockStatScreen(ctx, (Tscreen)x, TSF + i) || sockStatWidget(ctx, (Twidget)x, TSF + i) ||
+             sockStatObj(ctx, x, TSF + i);
         break;
-      case ggroup_magic_id:
-        ok = sockStatGroup((ggroup)x, TSF + i) || sockStatObj(x, TSF + i);
+      case Tgroup_class_byte:
+        ok = sockStatGroup(ctx, (Tgroup)x, TSF + i) || sockStatObj(ctx, x, TSF + i);
         break;
-      case menuitem_magic_id:
-        ok = sockStatMenuItem((menuitem)x, TSF + i) || sockStatRow((row)x, TSF + i) ||
-             sockStatObj(x, TSF + i);
+      case Tmenuitem_class_byte:
+        ok = sockStatMenuItem(ctx, (Tmenuitem)x, TSF + i) || sockStatRow(ctx, (Trow)x, TSF + i) ||
+             sockStatObj(ctx, x, TSF + i);
         break;
-      case menu_magic_id:
-        ok = sockStatMenu((menu)x, TSF + i) || sockStatObj(x, TSF + i);
+      case Tmenu_class_byte:
+        ok = sockStatMenu(ctx, (Tmenu)x, TSF + i) || sockStatObj(ctx, x, TSF + i);
         break;
-      case msgport_magic_id:
-        ok = sockStatMsgPort((msgport)x, TSF + i) || sockStatObj(x, TSF + i);
+      case Tmsgport_class_byte:
+        ok = sockStatMsgPort(ctx, (Tmsgport)x, TSF + i) || sockStatObj(ctx, x, TSF + i);
         break;
-      case mutex_magic_id:
-        ok = sockStatMutex((mutex)x, TSF + i) || sockStatObj(x, TSF + i);
+      case Tmutex_class_byte:
+        ok = sockStatMutex(ctx, (Tmutex)x, TSF + i) || sockStatObj(ctx, x, TSF + i);
         break;
-      case all_magic_id:
-        ok = sockStatAll((all)x, TSF + i) || sockStatObj(x, TSF + i);
+      case Tall_class_byte:
+        ok = sockStatAll(ctx, (Tall)x, TSF + i) || sockStatObj(ctx, x, TSF + i);
         break;
       default:
         ok = tfalse;
@@ -550,19 +537,22 @@ static void sockStat(obj x, udat n, const byte *in) {
     len = 2 * (j = i) * sizeof(udat) + 2 * sizeof(udat);
 
     for (i = 0; i < j; i++) {
-      if (TSF[i].type < TWS_highest)
+      if (TSF[i].type < TWS_highest) {
         len += TwinMagicData[TSF[i].type];
-      else if (TSF[i].type == TWS_obj) {
+      } else if (TSF[i].type == TWS_obj) {
         TSF[i].TWS_field_tobj = TSF[i].TWS_field_obj ? TSF[i].TWS_field_obj->Id : NOID;
         len += sizeof(tobj);
       } else {
-        if (!TSF[i].TWS_field_vecV)
+        if (!TSF[i].TWS_field_vecV) {
           TSF[i].TWS_field_vecL = 0;
+        }
         len += TSF[i].TWS_field_vecL + sizeof(uldat);
       }
     }
 
-    ok = sockReply(OK_MAGIC, len, NULL) && (data = RemoteWriteGetQueue(Slot, &q)) && q >= len;
+    ok = sockReply(ctx, OK_MAGIC, len, NULL)           //
+         && (data = RemoteWriteGetQueue(ctx.Slot, &q)) //
+         && q >= len;
 
     if (ok) {
       i = 0;
@@ -613,6 +603,7 @@ static void sockStat(obj x, udat n, const byte *in) {
 
     FreeMem(TSF);
   }
-  if (!ok)
-    sockReply(1, 0, NULL);
+  if (!ok) {
+    sockReply(ctx, 1, 0, NULL);
+  }
 }
