@@ -9,8 +9,10 @@
 #ifndef TWIN_TTERM_H
 #define TWIN_TTERM_H
 
-byte InitTerm(void);
+bool InitTerm(void);
 
-extern menu Term_Menu;
+Twindow OpenTerm(const char *arg0, const char *const *argv);
+
+extern Tmenu Term_Menu;
 
 #endif /* TWIN_TTERM_H */
