@@ -12,6 +12,8 @@
 
 #include "twin.h"
 
+#include <stdio.h> // rename()
+
 #ifdef TW_HAVE_PWD_H
 #include <pwd.h>
 #endif
@@ -35,21 +37,25 @@
 #include "alloc.h"
 #include "common.h"
 #include "data.h"
+#include "draw.h"
 #include "extreg.h"
+#include "log.h"
 #include "methods.h"
 #include "main.h"
-#include "draw.h"
+#include "hw.h"
+#include "privilege.h"
 #include "remote.h"
 #include "resize.h"
-#include "printk.h"
-#include "privilege.h"
-#include "util.h"
+#include "stl/string.h"
 #include "stl/vector.h"
-#include "hw.h"
+#include "util.h"
 
 #include <Tw/Twkeys.h>
 #include <Tutf/Tutf.h>
 
+String TmpDir;
+String SocketDir;
+
 void NormalizeTime(timevalue *Time) {
   if (Time->Fraction >= FullSEC || Time->Fraction < 0) {
     tany delta = Time->Fraction / FullSEC;
@@ -131,35 +137,38 @@ dat CmpTime(timevalue *T1, timevalue *T2) {
   return (dat)0;
 }
 
-static dat CmpCallTime(msgport m1, msgport m2) {
-  if ((!m1->FirstMsg) != (!m2->FirstMsg))
+static dat CmpCallTime(Tmsgport m1, Tmsgport m2) {
+  if ((!m1->Msgs.First) != (!m2->Msgs.First)) {
     /* one of the two doesn't have msgs */
-    return m1->FirstMsg ? (dat)-1 : (dat)1;
-  if ((!m1->WakeUp) != (!m2->WakeUp))
+    return m1->Msgs.First ? (dat)-1 : (dat)1;
+  }
+  if ((!m1->WakeUp) != (!m2->WakeUp)) {
     /* one doesn't need to be called */
     return m1->WakeUp ? (dat)-1 : (dat)1;
-  if ((!m1->WakeUp) && (!m2->WakeUp))
+  }
+  if ((!m1->WakeUp) && (!m2->WakeUp)) {
     return 0;
+  }
   return CmpTime(&m1->CallTime, &m2->CallTime);
 }
 
-byte Minimum(byte MaxIndex, const ldat *Array) {
-  byte i, MinIndex;
-  ldat Temp;
+byte Minimum(byte max_index, const ldat *array) {
+  byte i, min_i;
+  ldat temp;
 
-  if (!MaxIndex)
+  if (!max_index)
     return 0xFF;
 
-  Temp = Array[0];
-  MinIndex = 0;
+  temp = array[0];
+  min_i = 0;
 
-  for (i = 1; i < MaxIndex; i++) {
-    if (Array[i] < Temp) {
-      Temp = Array[i];
-      MinIndex = i;
+  for (i = 1; i < max_index; i++) {
+    if (array[i] < temp) {
+      temp = array[i];
+      min_i = i;
     }
   }
-  return MinIndex;
+  return min_i;
 }
 
 /*
@@ -218,42 +227,44 @@ void SetArgv0(char *const *argv, uldat argv_usable_len, const char *src) {
 }
 
 /*
- * move a msgport to the right place in an already sorted list,
+ * move a Tmsgport to the right place in an already sorted list,
  * ordering by CallTime
  */
-void SortMsgPortByCallTime(msgport Port) {
-  msgport other;
+void SortMsgPortByCallTime(Tmsgport Port) {
+  Tmsgport other;
   if ((other = Port->Next) && CmpCallTime(Port, other) > 0) {
     Port->Remove();
     do {
       other = other->Next;
     } while (other && CmpCallTime(Port, other) > 0);
-    if (other)
-      InsertMiddle(MsgPort, Port, All, other->Prev, other);
-    else
-      InsertLast(MsgPort, Port, All);
+    if (other) {
+      InsertMiddle(MsgPorts, Port, All, other->Prev, other);
+    } else {
+      InsertLast(MsgPorts, Port, All);
+    }
   } else if ((other = Port->Prev) && CmpCallTime(Port, other) < 0) {
     Port->Remove();
     do {
       other = other->Prev;
     } while (other && CmpCallTime(Port, other) < 0);
-    if (other)
-      InsertMiddle(MsgPort, Port, All, other, other->Next);
-    else
-      InsertFirst(MsgPort, Port, All);
+    if (other) {
+      InsertMiddle(MsgPorts, Port, All, other, other->Next);
+    } else {
+      InsertFirst(MsgPorts, Port, All);
+    }
   }
 }
 
 /*
- * sort the msgport list by growing CallTime
+ * sort the Tmsgport list by growing CallTime
  *
  * we use a bubble sort... no need to optimize to death this
  */
 void SortAllMsgPortsByCallTime(void) {
-  msgport Max, This, Port = All->FirstMsgPort;
-  msgport Start, End;
+  Tmsgport Max, This, Port = All->MsgPorts.First;
+  Tmsgport Start, End;
 
-  Start = End = (msgport)0;
+  Start = End = (Tmsgport)0;
 
   while (Port) {
     Max = This = Port;
@@ -278,32 +289,34 @@ void SortAllMsgPortsByCallTime(void) {
     if (!End)
       End = Max;
   }
-  All->FirstMsgPort = Start;
-  All->LastMsgPort = End;
+  All->MsgPorts.First = Start;
+  All->MsgPorts.Last = End;
 }
 
-byte SendControlMsg(msgport MsgPort, udat Code, udat Len, const char *Data) {
-  msg Msg;
-  event_control *Event;
+byte SendControlMsg(Tmsgport MsgPort, udat Code, udat Len, const char *Data) {
+  Tmsg msg;
+  event_control *event;
 
-  if (MsgPort && (Msg = New(msg)(msg_control, Len))) {
-    Event = &Msg->Event.EventControl;
-    Event->Code = Code;
-    Event->Len = Len;
-    CopyMem(Data, Event->Data, Len);
-    SendMsg(MsgPort, Msg);
+  if (MsgPort && (msg = Smsg::Create(msg_control, Len))) {
+    event = &msg->Event.EventControl;
+    event->Code = Code;
+    event->Len = Len;
+    CopyMem(Data, event->Data, Len);
+    SendMsg(MsgPort, msg);
 
     return ttrue;
   }
   return tfalse;
 }
 
-byte SelectionStore(uldat magic, const char mime[MAX_MIMELEN], Chars data) {
-  selection *Sel = All->Selection;
+bool SelectionStore(e_id magic, Chars mime, Chars data) {
+  Tselection Sel = All->Selection;
+  uldat len = Min2(mime.size(), TW_MAX_MIMELEN);
   if (mime) {
-    CopyMem(mime, Sel->MIME, MAX_MIMELEN);
-  } else {
-    memset(Sel->MIME, '\0', MAX_MIMELEN);
+    CopyMem(mime.data(), Sel->MIME, len);
+  }
+  if (len < TW_MAX_MIMELEN) {
+    memset(Sel->MIME + len, '\0', TW_MAX_MIMELEN - len);
   }
   Sel->Owner = NULL;
   Sel->Magic = magic;
@@ -323,7 +336,6 @@ byte SelectionAppendRunes(TRunes runes) {
   return All->Selection->Data.append(runes);
 }
 
-#define _SEL_MAGIC SEL_UTF8MAGIC
 #define _SelAppendNL() SelectionAppend(Chars("\n", 1));
 
 class Appender {
@@ -366,9 +378,8 @@ private:
   uldat spaces;
 };
 
-bool SetSelectionFromWindow(window w) {
-  ldat y;
-  uldat slen, i, spaces = 0;
+bool SetSelectionFromWindow(Twindow w) {
+  ldat y, slen, i;
   bool ok = true, w_useC = W_USE(w, USECONTENTS);
 
   if (!(w->State & WINDOW_DO_SEL)) {
@@ -378,9 +389,9 @@ bool SetSelectionFromWindow(window w) {
   if (!(w->State & WINDOW_ANYSEL) || w->YstSel > w->YendSel ||
       (w->YstSel == w->YendSel && w->XstSel > w->XendSel)) {
 
-    ok = SelectionStore(_SEL_MAGIC, NULL, Chars());
+    ok = SelectionStore(SEL_UTF8MAGIC, Chars(), Chars());
     if (ok) {
-      NeedHW |= NEEDSelectionExport;
+      NeedHW |= NeedSelectionExport;
     }
     return ok;
   }
@@ -419,7 +430,7 @@ bool SetSelectionFromWindow(window w) {
       w->XendSel = w->WLogic - 1;
     }
 
-    slen = w->WLogic;
+    slen = Max2(w->WLogic, 0);
     cells = w->USE.C.Contents + (w->YstSel + w->USE.C.HSplit) * slen;
     while (cells >= w->USE.C.TtyData->Split) {
       cells -= w->USE.C.TtyData->Split - w->USE.C.Contents;
@@ -433,8 +444,8 @@ bool SetSelectionFromWindow(window w) {
         slen = w->XendSel - w->XstSel + 1;
       }
       cells += w->XstSel;
-      ok = SelectionStore(_SEL_MAGIC, NULL, Chars());
-      for (i = slen; ok && i--; cells++) {
+      ok = SelectionStore(SEL_UTF8MAGIC, Chars(), Chars());
+      for (i = slen; ok && i-- > 0; cells++) {
         ok = appender.rune(TRUNE(*cells));
       }
       if (!ok) {
@@ -450,7 +461,7 @@ bool SetSelectionFromWindow(window w) {
         if (cells >= w->USE.C.TtyData->Split) {
           cells -= w->USE.C.TtyData->Split - w->USE.C.Contents;
         }
-        for (i = slen; ok && i--; cells++) {
+        for (i = slen; ok && i-- > 0; cells++) {
           ok = appender.rune(TRUNE(*cells));
         }
       }
@@ -463,14 +474,14 @@ bool SetSelectionFromWindow(window w) {
           cells -= w->USE.C.TtyData->Split - w->USE.C.Contents;
         }
         appender.nl();
-        for (i = slen = w->XendSel + 1; ok && i--; cells++) {
+        for (i = slen = w->XendSel + 1; ok && i-- > 0; cells++) {
           ok = appender.rune(TRUNE(*cells));
         }
       }
       if (ok) {
         ok = appender.flush();
         if (ok) {
-          NeedHW |= NEEDSelectionExport;
+          NeedHW |= NeedSelectionExport;
         }
       }
     } while (false);
@@ -481,24 +492,27 @@ bool SetSelectionFromWindow(window w) {
     return false;
   }
 
-  ok = SelectionStore(_SEL_MAGIC, NULL, Chars());
-  row r;
+  ok = SelectionStore(SEL_UTF8MAGIC, Chars(), Chars());
+  Trow r;
 
   /* Gap not supported! */
   y = w->YstSel;
   r = w->FindRow(y);
   if (r && r->Text) {
+    uldat xstSel = Min2u(r->Len, (uldat)w->XstSel);
     if (y < w->YendSel) {
-      slen = r->Len - w->XstSel;
+      slen = r->Len - xstSel;
     } else {
-      slen = Min2(r->Len, (uldat)w->XendSel + 1) - Min2(r->Len, (uldat)w->XstSel);
+      slen = Min2u(r->Len, (uldat)w->XendSel + 1) - xstSel;
     }
 
-    const trune *runes = r->Text + Min2(r->Len, (uldat)w->XstSel);
-    ok = ok && SelectionAppendRunes(TRunes(runes, slen));
+    if (slen > 0) {
+      const trune *runes = r->Text + Min2(r->Len, xstSel);
+      ok = ok && SelectionAppendRunes(TRunes(runes, slen));
+    }
   }
 
-  if (y < w->YendSel || !r || !r->Text || r->Len <= (uldat)w->XendSel) {
+  if (y < w->YendSel || !r || !r->Text || (ldat)r->Len <= w->XendSel) {
     ok = ok && _SelAppendNL();
   }
 
@@ -510,138 +524,326 @@ bool SetSelectionFromWindow(window w) {
   }
   if (w->YendSel > w->YstSel) {
     if (w->XendSel >= 0 && (r = w->FindRow(w->YendSel)) && r->Text) {
-      slen = Min2(r->Len, (uldat)w->XendSel + 1);
-      ok = ok && SelectionAppendRunes(TRunes(r->Text, slen));
+      slen = Min2((ldat)r->Len, w->XendSel + 1);
+      if (slen > 0) {
+        ok = ok && SelectionAppendRunes(TRunes(r->Text, slen));
+      }
     }
-    if (!r || !r->Text || r->Len <= (uldat)w->XendSel) {
+    if (!r || !r->Text || (ldat)r->Len <= w->XendSel) {
       ok = ok && _SelAppendNL();
     }
   }
   if (ok) {
-    NeedHW |= NEEDSelectionExport;
+    NeedHW |= NeedSelectionExport;
   }
   return ok;
 }
 
-byte CreateXTermMouseEvent(event_mouse *Event, byte buflen, char *buf) {
-  window w;
-  udat Flags;
-  udat Code = Event->Code;
-  dat x = Event->X, y = Event->Y;
-  byte len = 0;
-
-  if (!(w = (window)Event->W) || !IS_WINDOW(w) || !W_USE(w, USECONTENTS) || !w->USE.C.TtyData)
-    return len;
-
-  Flags = w->USE.C.TtyData->Flags;
-
-  if (Flags & TTY_REPORTMOUSE) {
-    /* new-style reporting */
-
-    /* when both TTY_REPORTMOUSE|TTY_REPORTMOUSE2 are set, also report motion */
-    if (buflen < 9 || (Code == MOVE_MOUSE && !(Flags & TTY_REPORTMOUSE2)))
-      /* buffer too small, or nothing to report */
-      return len;
+/* twterm-style mouse reporting */
+static byte CreateMouseEventTwterm(event_mouse *event, byte buflen, char *buf, const uldat flags) {
+  udat code = event->Code;
 
+  /* if TTY_REPORTMOUSE_MOVE is set, also report motion */
+  if (buflen < 9 || (code == MOVE_MOUSE && !(flags & TTY_REPORTMOUSE_MOVE))) {
+    /* buffer too small, or nothing to report */
+    return 0;
+  } else if (isPRESS(code)) {
     /* report also button just pressed as down */
-    if (isPRESS(Code))
-      Code |= HOLD_CODE(PRESS_N(Code));
-
-    CopyMem("\033[5M", buf, 4);
-    buf[4] = ' ' + ((Code & HOLD_ANY) >> HOLD_BITSHIFT);
-    buf[5] = '!' + (x & 0x7f);
-    buf[6] = '!' + ((x >> 7) & 0x7f);
-    buf[7] = '!' + (y & 0x7f);
-    buf[8] = '!' + ((y >> 7) & 0x7f);
-    len = 9;
-  } else if (Flags & TTY_REPORTMOUSE2) {
-    /* classic xterm-style reporting */
-
-    if (buflen < 6)
-      /* buffer too small! */
-      return len;
-
-    CopyMem("\033[M", buf, 3);
-
-    if (isSINGLE_PRESS(Code))
-      switch (Code & PRESS_ANY) {
-      case PRESS_LEFT:
-        buf[3] = ' ';
-        break;
-      case PRESS_MIDDLE:
-        buf[3] = '!';
-        break;
-      case PRESS_RIGHT:
-        buf[3] = '\"';
-        break;
-        /* WHEEL_REV and WHEEL_FWD supported only at release */
-      }
-    else if (isRELEASE(Code)) {
-      switch (Code & RELEASE_ANY) {
-#ifdef HOLD_WHEEL_REV
-      case RELEASE_WHEEL_REV:
-        buf[3] = '`';
-        break;
-#endif
-#ifdef HOLD_WHEEL_FWD
-      case RELEASE_WHEEL_FWD:
-        buf[3] = 'a';
-        break;
-#endif
-      default:
-        buf[3] = '#';
-        break;
-      }
-    } else
-      return len;
-
-    buf[4] = '!' + x;
-    buf[5] = '!' + y;
-    len = 6;
+    code |= HOLD_CODE(PRESS_N(code));
   }
-  return len;
+  CopyMem("\033[5M", buf, 4);
+
+  const dat x = event->X, y = event->Y;
+
+  buf[4] = ' ' + ((code & HOLD_ANY) >> HOLD_BITSHIFT);
+  buf[5] = '!' + (x & 0x7f);
+  buf[6] = '!' + ((x >> 7) & 0x7f);
+  buf[7] = '!' + (y & 0x7f);
+  buf[8] = '!' + ((y >> 7) & 0x7f);
+  return 9;
 }
 
-void ResetBorderPattern(void) {
-  msgport MsgP;
-  widget w;
+/* xterm-style mouse reporting */
+static byte CreateMouseEventXterm(event_mouse *event, byte buflen, char *buf, const uldat flags) {
 
-  for (MsgP = All->FirstMsgPort; MsgP; MsgP = MsgP->Next) {
-    for (w = MsgP->FirstW; w; w = w->O_Next) {
-      if (IS_WINDOW(w))
-        ((window)w)->BorderPattern[0] = ((window)w)->BorderPattern[1] = NULL;
+  if (buflen < 6) {
+    /* buffer too small! */
+    return 0;
+  }
+  CopyMem("\033[M", buf, 4);
+  const udat code = event->Code;
+  byte ch = 0;
+
+  if (isPRESS(code)) {
+    const udat press = code & PRESS_ANY;
+    if (press == PRESS_LEFT) {
+      ch = ' ';
+    } else if (press == PRESS_MIDDLE) {
+      ch = '!';
+    } else if (press == PRESS_RIGHT) {
+      ch = '"';
+    } else {
+      /* WHEEL_REV and WHEEL_FWD are reported only at release */
+      return 0;
+    }
+  } else if (isRELEASE(code)) {
+    const udat release = code & RELEASE_ANY;
+    if (release == RELEASE_WHEEL_REV) {
+      ch = '`';
+    } else if (release == RELEASE_WHEEL_FWD) {
+      ch = 'a';
+    } else {
+      ch = '#';
+    }
+  } else if (code == MOVE_MOUSE) {
+    if (!(flags & TTY_REPORTMOUSE_MOVE)) {
+      return 0;
+    }
+    /* also report mouse move */
+    ch = '@';
+  } else {
+    if (!(flags & TTY_REPORTMOUSE_DRAG)) {
+      return 0;
+    }
+    /* also report mouse drag */
+    const udat hold = code & HOLD_ANY;
+    if (hold == HOLD_LEFT) {
+      ch = '@';
+    } else if (hold == HOLD_MIDDLE) {
+      ch = 'A';
+    } else if (hold == HOLD_RIGHT) {
+      ch = 'B';
+    } else {
+      ch = '@';
+    }
+  }
+  buf[3] = ch;
+  const dat x = event->X, y = event->Y;
+  buf[4] = '!' + Clamp(0, x, 222);
+  buf[5] = '!' + Clamp(0, y, 222);
+  return 6;
+}
+
+static char *PrintUdat100k(char *dst, udat n) {
+  if (n < 10) {
+    dst[0] = n + '0';
+    return dst + 1;
+  }
+  if (n < 100) {
+    dst[0] = (n / 10) + '0';
+    dst[1] = (n % 10) + '0';
+    return dst + 2;
+  }
+  byte len;
+  if (n < 1000) {
+    len = 3;
+  } else if (n < 10000) {
+    len = 4;
+  } else {
+    len = 5;
+  }
+  char *ret = dst + len;
+  while (len--) {
+    dst[len] = (n % 10) + '0';
+    n /= 10;
+  }
+  return ret;
+}
+
+/* xterm-style mouse reporting, decimal coordinates */
+static byte CreateMouseEventXtermDecimal(event_mouse *event, byte buflen, char *buf,
+                                         const uldat flags) {
+  if (buflen < 18) {
+    /* buffer too small! */
+    return 0;
+  }
+  CopyMem("\033[<", buf, 4);
+
+  char *p = buf + 3;
+  const udat code = event->Code;
+
+  if (isPRESS(code)) {
+    const udat press = code & PRESS_ANY;
+    if (press == PRESS_LEFT) {
+      *p++ = '0';
+    } else if (press == PRESS_MIDDLE) {
+      *p++ = '1';
+    } else if (press == PRESS_RIGHT) {
+      *p++ = '2';
+    } else if (press == PRESS_WHEEL_REV) {
+      *p++ = '6';
+      *p++ = '4';
+    } else if (press == PRESS_WHEEL_FWD) {
+      *p++ = '6';
+      *p++ = '5';
+    } else {
+      return 0;
+    }
+  } else if (isRELEASE(code)) {
+    const udat release = code & RELEASE_ANY;
+    if (release == RELEASE_LEFT) {
+      *p++ = '0';
+    } else if (release == RELEASE_MIDDLE) {
+      *p++ = '1';
+    } else if (release == RELEASE_RIGHT) {
+      *p++ = '2';
+    } else {
+      // RELEASE_WHEEL_REV and RELEASE_WHEEL_FWD are not reported
+      return 0;
+    }
+  } else if (code == MOVE_MOUSE) {
+    if (!(flags & TTY_REPORTMOUSE_MOVE)) {
+      return 0;
+    }
+    /* also report mouse move */
+    *p++ = '3';
+    *p++ = '5';
+  } else {
+    if (!(flags & TTY_REPORTMOUSE_DRAG)) {
+      return 0;
+    }
+    *p++ = '3';
+    /* also report mouse dragging */
+    const udat hold = code & HOLD_ANY;
+    if (hold == HOLD_LEFT) {
+      *p++ = '2';
+    } else if (hold == HOLD_MIDDLE) {
+      *p++ = '3';
+    } else if (hold == HOLD_RIGHT) {
+      *p++ = '4';
+    } else {
+      *p++ = '2';
+    }
+  }
+  *p++ = ';';
+  const udat x = 1 + Clamp(0, event->X, 99998);
+  p = PrintUdat100k(p, x);
+  *p++ = ';';
+  const udat y = 1 + Clamp(0, event->Y, 99998);
+  p = PrintUdat100k(p, y);
+  *p++ = isRELEASE(code) ? 'm' : 'M';
+  return p - buf;
+}
+
+/* X10-style mouse reporting */
+static byte CreateMouseEventX10(event_mouse *event, byte buflen, char *buf, const uldat flags) {
+
+  const udat code = event->Code;
+
+  if (buflen < 6 || !isPRESS(code)) {
+    return 0;
+  }
+  CopyMem("\033[M", buf, 4);
+  char ch;
+  switch (code & PRESS_ANY) {
+  case PRESS_LEFT:
+    ch = ' ';
+    break;
+  case PRESS_MIDDLE:
+    ch = '!';
+    break;
+  case PRESS_RIGHT:
+    ch = '!';
+    break;
+  default:
+    /* WHEEL_REV and WHEEL_FWD are not reported */
+    return 0;
+  }
+  buf[3] = ch;
+  const dat x = event->X, y = event->Y;
+  buf[4] = '!' + Clamp(0, x, 222);
+  buf[5] = '!' + Clamp(0, y, 222);
+  return 6;
+}
+
+byte CreateMouseEvent(event_mouse *event, byte buflen, char *buf) {
+  Twindow w = (Twindow)event->W;
+
+  if (!w || !IS_WINDOW(w) || !W_USE(w, USECONTENTS) || !w->USE.C.TtyData) {
+    return 0;
+  }
+  const uldat flags = w->USE.C.TtyData->Flags;
+
+  switch (flags & TTY_REPORTMOUSE_STYLE) {
+  case TTY_REPORTMOUSE_TWTERM:
+    return CreateMouseEventTwterm(event, buflen, buf, flags);
+  case TTY_REPORTMOUSE_XTERM:
+    if (flags & TTY_REPORTMOUSE_DECIMAL) {
+      return CreateMouseEventXtermDecimal(event, buflen, buf, flags);
+    } else {
+      return CreateMouseEventXterm(event, buflen, buf, flags);
+    }
+  case TTY_REPORTMOUSE_X10:
+    return CreateMouseEventX10(event, buflen, buf, flags);
+  default:
+    return 0;
+  }
+}
+
+void closeAllFds(int tty_fd_to_dup) {
+  // do not assume all other file descriptors are set to fcntl(fd, F_SETFD, FD_CLOEXEC)
+  int i;
+  if (tty_fd_to_dup >= 0) {
+    for (i = 0; i <= 2; i++) {
+      if (i != tty_fd_to_dup) {
+        close(i);
+        dup2(tty_fd_to_dup, i);
+      }
+    }
+    if (tty_fd_to_dup > 2) {
+      close(tty_fd_to_dup);
+    }
+  } else {
+    i = 0;
+  }
+  int last_successful_close = i - 1;
+  for (; i < INT_MAX && i - 10 < last_successful_close; i++) {
+    if (close(i) >= 0) {
+      last_successful_close = i;
     }
   }
 }
 
-static gadget _PrevGadget(gadget G) {
-  while (G->Prev) {
-    G = (gadget)G->Prev;
-    if (IS_GADGET(G))
-      return (gadget)G;
+void ResetBorderPattern(void) {
+  Tmsgport MsgP;
+  Twidget w;
+
+  for (MsgP = All->MsgPorts.First; MsgP; MsgP = MsgP->Next) {
+    for (w = MsgP->Widgets.First; w; w = w->O_Next) {
+      if (IS_WINDOW(w))
+        ((Twindow)w)->BorderPattern[0] = ((Twindow)w)->BorderPattern[1] = NULL;
+    }
   }
-  return (gadget)G->Prev;
 }
 
-static gadget _NextGadget(gadget G) {
-  while (G->Next) {
-    G = (gadget)G->Next;
+static Tgadget _PrevGadget(Tgadget G) {
+  while (G->Prev) {
+    G = (Tgadget)G->Prev;
     if (IS_GADGET(G))
-      return (gadget)G;
+      return (Tgadget)G;
   }
-  return (gadget)G->Next;
+  return (Tgadget)G->Prev;
+}
+
+static Tgadget _NextGadget(Tgadget G) {
+  while (G->Next) {
+    G = (Tgadget)G->Next;
+    if (IS_GADGET(G))
+      return (Tgadget)G;
+  }
+  return (Tgadget)G->Next;
 }
 
 /* handle common keyboard actions like cursor moving and button navigation */
-void FallBackKeyAction(window w, event_keyboard *EventK) {
+void FallBackKeyAction(Twindow w, event_keyboard *EventK) {
   ldat NumRow, OldNumRow;
-  gadget G, H;
+  Tgadget G, H;
 
-  if ((G = (gadget)w->SelectW) && IS_GADGET(G))
+  if ((G = (Tgadget)w->SelectW) && IS_GADGET(G))
     switch (EventK->Code) {
     case TW_Escape:
       UnPressGadget(G, tfalse);
-      w->SelectW = (widget)0;
+      w->SelectW = (Twidget)0;
       break;
     case TW_Return:
       UnPressGadget(G, ttrue);
@@ -652,7 +854,7 @@ void FallBackKeyAction(window w, event_keyboard *EventK) {
       if ((H = _PrevGadget(G))) {
         if (!(G->Flags & GADGETFL_TOGGLE))
           UnPressGadget(G, tfalse);
-        w->SelectW = (widget)H;
+        w->SelectW = (Twidget)H;
         PressGadget(H);
       }
       break;
@@ -662,16 +864,16 @@ void FallBackKeyAction(window w, event_keyboard *EventK) {
       if ((H = _NextGadget(G))) {
         if (!(G->Flags & GADGETFL_TOGGLE))
           UnPressGadget(G, tfalse);
-        w->SelectW = (widget)H;
+        w->SelectW = (Twidget)H;
         PressGadget(H);
       }
       break;
     default:
       break;
     }
-  else if ((G = (gadget)w->FirstW) && IS_GADGET(G)) {
+  else if ((G = (Tgadget)w->Widgets.First) && IS_GADGET(G)) {
     PressGadget(G);
-    w->SelectW = (widget)G;
+    w->SelectW = (Twidget)G;
   } else
     switch (EventK->Code) {
     case TW_Up:
@@ -685,11 +887,11 @@ void FallBackKeyAction(window w, event_keyboard *EventK) {
           NumRow = OldNumRow - (ldat)1;
         w->CurY = NumRow;
         if (w->Flags & WINDOWFL_ROWS_SELCURRENT)
-          DrawLogicWidget((widget)w, (ldat)0, OldNumRow, (ldat)TW_MAXDAT - (ldat)2, OldNumRow);
+          DrawLogicWidget((Twidget)w, (ldat)0, OldNumRow, (ldat)TW_MAXDAT - (ldat)2, OldNumRow);
       } else
         w->CurY = NumRow = w->HLogic - (ldat)1;
       if (w->Flags & WINDOWFL_ROWS_SELCURRENT)
-        DrawLogicWidget((widget)w, (ldat)0, NumRow, (ldat)TW_MAXDAT - (ldat)2, NumRow);
+        DrawLogicWidget((Twidget)w, (ldat)0, NumRow, (ldat)TW_MAXDAT - (ldat)2, NumRow);
       UpdateCursor();
       break;
     case TW_Down:
@@ -703,11 +905,11 @@ void FallBackKeyAction(window w, event_keyboard *EventK) {
           NumRow = OldNumRow + (ldat)1;
         w->CurY = NumRow;
         if (w->Flags & WINDOWFL_ROWS_SELCURRENT)
-          DrawLogicWidget((widget)w, (ldat)0, OldNumRow, (ldat)TW_MAXDAT - (ldat)2, OldNumRow);
+          DrawLogicWidget((Twidget)w, (ldat)0, OldNumRow, (ldat)TW_MAXDAT - (ldat)2, OldNumRow);
       } else
         w->CurY = NumRow = (ldat)0;
       if (w->Flags & WINDOWFL_ROWS_SELCURRENT)
-        DrawLogicWidget((widget)w, (ldat)0, NumRow, (ldat)TW_MAXDAT - (ldat)2, NumRow);
+        DrawLogicWidget((Twidget)w, (ldat)0, NumRow, (ldat)TW_MAXDAT - (ldat)2, NumRow);
       UpdateCursor();
       break;
     case TW_Left:
@@ -849,7 +1051,7 @@ int unixFd;
 uldat unixSlot;
 
 static void TWDisplayIO(int fd, uldat slot) {
-  struct sockaddr_un un_addr;
+  struct sockaddr_un un_addr = {};
   socklen_t len = sizeof(un_addr);
 
   if ((fd = accept(fd, (struct sockaddr *)&un_addr, &len)) >= 0) {
@@ -859,11 +1061,111 @@ static void TWDisplayIO(int fd, uldat slot) {
 
 static char envTWD[] = "TWDISPLAY=\0\0\0\0\0";
 
-const char *TmpDir(void) {
-  const char *tmp = getenv("TMPDIR");
-  if (tmp == NULL)
-    tmp = "/tmp";
-  return tmp;
+static bool moveOldTwinrcFile(void);
+
+static bool initHOME(void) {
+  const char *home = getenv("HOME");
+  if (!home) {
+    log(ERROR) << "twin: required environment variable $HOME is not set. Aborting.\n";
+    return false;
+  }
+  if (!HOME.format(Chars::from_c(home))) { // also append final '\0' but do not count it
+    log(ERROR) << "twin: out of memory! Aborting.\n";
+    return false;
+  }
+  if (!moveOldTwinrcFile()) {
+    return false;
+  }
+  return true;
+}
+
+// create directory and all its parent directories.
+// temporarily modifies path, restores it before returning
+static void makeDirectories(char *path) {
+  if (!path || !path[0]) {
+    return;
+  }
+  char *slash = path;
+  char *end = path + strlen(path);
+  while ((slash = (char *)memchr(slash + 1, '/', end - (slash + 1))) != NULL) {
+    slash[0] = '\0';
+    (void)mkdir(path, 0700);
+    slash[0] = '/';
+  }
+  (void)mkdir(path, 0700);
+}
+
+bool moveOldTwinrcFile(void) {
+  String oldPath;
+  if (!oldPath.format(HOME, "/.twinrc")) {
+    log(ERROR) << "twin: out of memory! Aborting.\n";
+    return false;
+  }
+  struct stat buf;
+  if (stat(oldPath.data(), &buf) != 0) {
+    return true;
+  }
+  String newPath;
+  if (!newPath.format(HOME, "/.config/twin/twinrc")) {
+    log(ERROR) << "twin: out of memory! Aborting.\n";
+    return false;
+  }
+  newPath[newPath.size() - 7] = '\0';
+  makeDirectories(newPath.data());
+  newPath[newPath.size() - 7] = '/';
+  if (rename(oldPath.data(), newPath.data()) != 0) {
+    log(ERROR)
+        << "twin: failed to move configuration file from old (and no longer supported) location "
+        << oldPath << "\n      to the new location " << newPath << ": "
+        << Chars::from_c(strerror(errno));
+    return false;
+  }
+  log(WARNING) << "twin: moved configuration file from old (and no longer supported) location "
+               << oldPath << "\n      to the new location " << newPath;
+  return true;
+}
+
+/**
+ * initialize TmpDir to the absolutep path of temporary directory:
+ * if getenv("TMPDIR") is not null, use it;
+ * otherwise use "/tmp"
+ * on out-of-memory, return false
+ */
+static bool initTmpDir(void) {
+  const char *dir = getenv("TMPDIR");
+  if (dir == NULL) {
+    dir = "/tmp";
+  }
+  if (TmpDir.assign(dir, 1 + strlen(dir))) { // also append final '\0'
+    TmpDir.pop_back();                       // but do not count it
+    return true;
+  }
+  log(ERROR) << "twin: out of memory! Aborting.\n";
+  return false;
+}
+
+/**
+ * initialize SOCKET_DIR to the directory where to create unix domain socket to listen on:
+ * if getenv("XDG_STATE_HOME") is not null, use it + "/twin"
+ * otherwise use HOME + "/.local/state/twin"
+ * on out-of-memory, return false
+ */
+/** return getenv("XDG_STATE_HOME") + "/twin", getenv("HOME") + "/.local/state/twin"  */
+static bool initSocketDir(void) {
+  const char *env = getenv("XDG_STATE_HOME");
+  if (env != NULL && SocketDir.format(Chars::from_c(env), Chars("/twin"))) {
+    return true;
+  } else if (SocketDir.format(HOME, Chars("/.local/state/twin"))) {
+    return true;
+  } else {
+    SocketDir.clear();
+    log(ERROR) << "twin: out of memory! Aborting.\n";
+    return false;
+  }
+}
+
+bool InitGlobalVariables(void) {
+  return initHOME() && initTmpDir() && initSocketDir();
 }
 
 udat CopyToSockaddrUn(const char *src, struct sockaddr_un *addr, udat pos) {
@@ -881,15 +1183,18 @@ static struct sockaddr_un addr_unix;
 static const char *fullTWD = addr_unix.sun_path;
 static char twd[12];
 
-/* set TWDISPLAY and create /tmp/.Twin:<x> */
-byte InitTWDisplay(void) {
+/*
+ * create unix domain socket $HOME/.local/state/twin/socket:HHH
+ * create symbolic link /tmp/.Twin:HHH pointing to the former
+ * set TWDISPLAY to :HHH
+ */
+bool InitTWDisplay(void) {
   char *arg0;
   int fd = NOFD;
   unsigned short i;
   udat len;
   byte ok;
 
-  HOME = getenv("HOME");
   memset(&addr_unix, 0, sizeof(addr_unix));
 
   if ((unixFd = socket(AF_UNIX, SOCK_STREAM, 0)) >= 0) {
@@ -897,9 +1202,9 @@ byte InitTWDisplay(void) {
     addr_unix.sun_family = AF_UNIX;
 
     for (i = 0; i < 0x1000; i++) {
-      sprintf(twd, ":%hx", i);
+      snprintf(twd, sizeof(twd), ":%hx", i);
 
-      len = CopyToSockaddrUn(TmpDir(), &addr_unix, 0);
+      len = CopyToSockaddrUn(TmpDir.data(), &addr_unix, 0);
       len = CopyToSockaddrUn("/.Twin", &addr_unix, len);
       len = CopyToSockaddrUn(twd, &addr_unix, len);
 
@@ -911,7 +1216,7 @@ byte InitTWDisplay(void) {
           if (connect(fd, (struct sockaddr *)&addr_unix, sizeof(addr_unix)) >= 0) {
             /*
              * server is alive, try to grab another TWDISPLAY.
-             * also, we must junk `fd' since SOCK_STREAM sockets
+             * also, we must close `fd' since SOCK_STREAM sockets
              * can connect() only once
              */
             close(fd);
@@ -927,7 +1232,7 @@ byte InitTWDisplay(void) {
              *
              * Trying to delete a /tmp/.Twin:<x> entry we cannot
              * connect to wreaks havoc if you mix this twin server
-             * with older ones, but having two different servers
+             * with older ones, but having two different server versions
              * installed on one system should be rare enough.
              */
             ok = bind(unixFd, (struct sockaddr *)&addr_unix, sizeof(addr_unix)) >= 0;
@@ -945,19 +1250,24 @@ byte InitTWDisplay(void) {
             TWDisplay = twd;
             lenTWDisplay = strlen(TWDisplay);
             CopyMem(TWDisplay, envTWD + 10, lenTWDisplay);
+            /**
+             * cannot set environment variable TERM=xterm-256color here:
+             * it confuses libgpm, used by --hw=tty
+             *
+             * solution: set it only in child processes, see spawnInWindow() in pty.cpp
+             */
 #if defined(TW_HAVE_SETENV)
             setenv("TWDISPLAY", TWDisplay, 1);
-            setenv("TERM", "linux", 1);
 #elif defined(TW_HAVE_PUTENV)
             putenv(envTWD);
-            putenv("TERM=linux");
 #endif
-            if ((arg0 = (char *)AllocMem(strlen(TWDisplay) + 6))) {
-              sprintf(arg0, "twin %s", TWDisplay);
+            size_t arg0_len = strlen(TWDisplay) + 6;
+            if ((arg0 = (char *)AllocMem(arg0_len))) {
+              snprintf(arg0, arg0_len, "twin %s", TWDisplay);
               SetArgv0(main_argv, main_argv_usable_len, arg0);
               FreeMem(arg0);
             }
-            return ttrue;
+            return true;
           }
         } else
           Error(SYSERROR);
@@ -968,14 +1278,16 @@ byte InitTWDisplay(void) {
   if (fd != NOFD)
     close(fd);
 
-  CopyToSockaddrUn(TmpDir(), &addr_unix, 0);
+  CopyToSockaddrUn(TmpDir.data(), &addr_unix, 0);
   arg0 = addr_unix.sun_path;
 
-  printk("twin: failed to create any " SS "/.Twin* socket: " SS "\n", addr_unix.sun_path, Errstr);
-  printk("      possible reasons: either " SS " not writable, or all TWDISPLAY already in use,\n"
-         "      or too many stale " SS "/.Twin* sockets. Aborting.\n",
-         arg0, arg0);
-  return tfalse;
+  log(ERROR) << "twin: failed to create any " << Chars::from_c(addr_unix.sun_path)
+             << "/.Twin* socket: " << Errstr << "\n      possible reasons: either "
+             << Chars::from_c(arg0)
+             << " is not writable, or all TWDISPLAY are already in use,\n"
+                "      or too many stale "
+             << Chars::from_c(arg0) << "/.Twin* sockets. Aborting.\n";
+  return false;
 }
 
 /* unlink /tmp/.Twin<TWDISPLAY> */
@@ -990,7 +1302,7 @@ static gid_t tty_grgid;
 byte CheckPrivileges(void) {
   Uid = getuid();
   EUid = geteuid();
-  tty_grgid = get_tty_grgid();
+  tty_grgid = getTtyGrgid();
 
   if (GainRootPrivileges() >= 0)
     Privilege = suidroot;
@@ -999,42 +1311,49 @@ byte CheckPrivileges(void) {
   else
     Privilege = none;
 
-  DropPrivileges();
+  dropPrivileges();
 
   return Privilege;
 }
 
-void GainPrivileges(void) {
+void gainPrivileges(void) {
   if (Privilege == suidroot)
     GainRootPrivileges();
   else if (Privilege == sgidtty)
-    GainGroupPrivileges(get_tty_grgid());
+    GainGroupPrivileges(getTtyGrgid());
 }
 
-static void SetEnvs(struct passwd *p) {
+static bool SetEnvs(struct passwd *p) {
   char buf[TW_BIGBUFF];
 
-  chdir(HOME = p->pw_dir);
+  chdir(p->pw_dir);
+  if (!HOME.assign(p->pw_dir, 1 + strlen(p->pw_dir))) { // also append final '\0'
+    return false;
+  }
+  // keep final '\0' but do not count it
+  HOME.pop_back();
+
 #if defined(TW_HAVE_SETENV)
-  setenv("HOME", HOME, 1);
+  setenv("HOME", HOME.data(), 1);
   setenv("SHELL", p->pw_shell, 1);
   setenv("LOGNAME", p->pw_name, 1);
-  sprintf(buf, "/var/mail/%.*s", (int)(TW_BIGBUFF - 11), p->pw_name);
+  snprintf(buf, TW_BIGBUFF, "/var/mail/%.*s", (int)(TW_BIGBUFF - 11), p->pw_name);
   setenv("MAIL", buf, 1);
 #elif defined(TW_HAVE_PUTENV)
-  sprintf(buf, "HOME=%.*s", (int)(TW_BIGBUFF - 6), HOME);
-  putenv(buf);
-  sprintf(buf, "SHELL=%.*s", (int)(TW_BIGBUFF - 7), p->pw_shell);
-  putenv(buf);
-  sprintf(buf, "LOGNAME=%.*s", (int)(TW_BIGBUFF - 9), p->pw_name);
-  putenv(buf);
-  sprintf(buf, "MAIL=/var/mail/%.*s", (int)(TW_BIGBUFF - 16) p->pw_name);
-  putenv(buf);
+  snprintf(buf, TW_BIGBUFF, "HOME=%.*s", (int)(TW_BIGBUFF - 6), HOME.data());
+  putenv(CloneStr(buf));
+  snprintf(buf, TW_BIGBUFF, "SHELL=%.*s", (int)(TW_BIGBUFF - 7), p->pw_shell);
+  putenv(CloneStr(buf));
+  snprintf(buf, TW_BIGBUFF, "LOGNAME=%.*s", (int)(TW_BIGBUFF - 9), p->pw_name);
+  putenv(CloneStr(buf));
+  snprintf(buf, TW_BIGBUFF, "MAIL=/var/mail/%.*s", (int)(TW_BIGBUFF - 16), p->pw_name);
+  putenv(CloneStr(buf));
 #endif
+  return true;
 }
 
 byte SetServerUid(uldat uid, byte privileges) {
-  msgport WM_MsgPort;
+  Tmsgport WM_MsgPort;
   struct passwd *p;
   byte ok = tfalse;
 
@@ -1061,76 +1380,72 @@ byte SetServerUid(uldat uid, byte privileges) {
         }
         if (ok && (uid == 0 || CheckPrivileges() == privileges)) {
           flag_secure = 0;
-          SetEnvs(p);
-          if ((ok = Ext(Socket, InitAuth)())) {
+          ok = SetEnvs(p);
+          if (ok && (ok = Ext(Socket, InitAuth)())) {
             /*
-             * it's time to execute .twenvrc.sh and read its output to set
+             * it's time to execute ~/.config/twin/twenvrc.sh and read its output to set
              * environment variables (mostly useful for twdm)
              */
             RunTwEnvRC();
 
-            /* tell the WM to restart itself (so that it reads user's .twinrc) */
+            /* tell the WM to restart itself (so that it reads user's ~/.config/twin/twinrc) */
             SendControlMsg(WM_MsgPort, MSG_CONTROL_OPEN, 0, NULL);
             return ttrue;
           }
-        } else
+        } else {
           ok = tfalse;
+        }
 
         if (!ok) {
           flag_secure = 1;
-          if (setuid(0) < 0 || setgid(0) < 0 || chown(fullTWD, 0, 0) < 0) {
+          if (setuid(0) < 0 || setgid(0) < 0 || chown(fullTWD, 0, 0) < 0 || !SetEnvs(getpwuid(0))) {
             /* tried to recover, but screwed up uids too badly. */
-            printk("twin: failed switching to uid %u: " SS "\n", uid, strerror(errno));
-            printk("twin: also failed to recover. Quitting NOW!\n");
+            log(ERROR) << "twin: failed switching to uid " << uid << ": "
+                       << Chars::from_c(strerror(errno))
+                       << "\ntwin: also failed to recover. Quitting NOW!\n";
             Quit(0);
           }
-          SetEnvs(getpwuid(0));
         }
       }
-      printk("twin: failed switching to uid %u: " SS "\n", uid, strerror(errno));
+      log(ERROR) << "twin: failed switching to uid " << uid << ": "
+                 << Chars::from_c(strerror(errno)) << "\n";
     }
   } else
-    printk("twin: SetServerUid() can be called only if started by root with \"-secure\".\n");
+    log(ERROR) << "twin: SetServerUid() can be called only if started by root with \"-secure\".\n";
   return tfalse;
 }
 
-/*
- * search for a file relative to HOME, to PKG_LIBDIR or as path
- *
- * this for example will search "foo"
- * as "${HOME}/foo", "${PKG_LIBDIR}/system.foo" or plain "foo"
- */
-char *FindFile(const char *name, uldat *fsize) {
-  const char *prefix[3], *infix[3];
-  char *path;
-  const char *dir;
-  int i, min_i, max_i, len, nlen = strlen(name);
+char *FindConfigFile(const char *name, uldat *file_size) {
+  const Chars prefix[4] = {
+      Chars::from_c(getenv("XDG_CONFIG_HOME")),
+      HOME,
+      confdir,
+      Chars("."),
+  };
+  const Chars infix[4] = {
+      Chars("/twin/"),
+      Chars("/.config/twin/"),
+      Chars("/"),
+      Chars("/"),
+  };
+  unsigned min_i, max_i;
+  if (flag_secure)
+    min_i = max_i = 2; /* only confdir */
+  else
+    min_i = 0, max_i = 3;
+
+  String path;
+  Chars cname = Chars::from_c(name);
   struct stat buf;
 
-  prefix[0] = HOME;
-  infix[0] = (HOME && *HOME) ? "/" : "";
-  prefix[1] = pkg_libdir;
-  infix[1] = "/system";
-  prefix[2] = "";
-  infix[2] = "";
-
-  if (flag_secure)
-    min_i = max_i = 1; /* only pkg_libdir */
-  else
-    min_i = 0, max_i = 2;
-
-  for (i = min_i; i <= max_i; i++) {
-    if (!(dir = prefix[i]))
-      continue;
-    len = strlen(dir) + strlen(infix[i]);
-    if ((path = (char *)AllocMem(len + nlen + 2))) {
-      sprintf(path, "%s%s%s", dir, infix[i], name);
-      if (stat(path, &buf) == 0) {
-        if (fsize)
-          *fsize = buf.st_size;
-        return path;
+  for (unsigned i = min_i; i <= max_i; i++) {
+    if (prefix[i] && path.format(prefix[i], infix[i], cname)) {
+      if (stat(path.data(), &buf) == 0) {
+        if (file_size) {
+          *file_size = buf.st_size;
+        }
+        return path.release();
       }
-      FreeMem(path);
     }
   }
   return NULL;
@@ -1160,7 +1475,7 @@ static void ReadTwEnvRC(int infd) {
       *eq++ = '\0';
       setenv(p, eq, 1);
 #elif defined(TW_HAVE_PUTENV)
-      putenv(p);
+      putenv(CloneStr(p));
 #endif
       p = q;
     }
@@ -1169,19 +1484,18 @@ static void ReadTwEnvRC(int infd) {
       /* line too long! */
       left = 0;
 
-    memmove(buff, p, left);
+    std::memmove(buff, p, left);
     p = buff + left;
     left = TW_BIGBUFF - left;
   }
 }
 
 /*
- * execute .twenvrc.sh <dummy> and read its output to set
+ * execute ~/.config/twin/twenvrc.sh and read its output to set
  * environment variables (mostly useful for twdm)
  */
 void RunTwEnvRC(void) {
-  char *path;
-  uldat len;
+  char *path = NULL;
   int fds[2];
 
   if (flag_envrc != 1)
@@ -1190,13 +1504,14 @@ void RunTwEnvRC(void) {
   if (flag_secure == 0) {
     flag_envrc = 0;
 
-    if ((path = FindFile(".twenvrc.sh", &len))) {
+    if ((path = FindConfigFile("twenvrc.sh", NULL))) {
       if ((pipe(fds) >= 0)) {
         switch (fork()) {
         case -1: /* error */
           close(fds[0]);
           close(fds[1]);
-          printk("twin: RunTwEnvRC(): fork() failed: " SS "\n", strerror(errno));
+          log(ERROR) << "twin: RunTwEnvRC(): fork() failed: " << Chars::from_c(strerror(errno))
+                     << "\n";
           break;
         case 0: /* child */
           close(fds[0]);
@@ -1207,8 +1522,10 @@ void RunTwEnvRC(void) {
           }
           close(1);
           dup2(0, 1);
-          execl(path, path, "dummy", NULL);
-          exit(0);
+          execlp("bash", "bash", "-i", "-l", path, NULL);
+          // if bash is not found, try plain sh
+          execlp("sh", "sh", path, NULL);
+          exit(1);
           break;
         default: /* parent */
           close(fds[1]);
@@ -1217,11 +1534,13 @@ void RunTwEnvRC(void) {
           break;
         }
       } else
-        printk("twin: RunTwEnvRC(): pipe() failed: " SS "\n", strerror(errno));
+        log(ERROR) << "twin: RunTwEnvRC(): pipe() failed: " << Chars::from_c(strerror(errno))
+                   << "\n";
     } else
-      printk("twin: RunTwEnvRC(): .twenvrc.sh: File not found\n", strerror(errno));
+      log(ERROR) << "twin: RunTwEnvRC(): twenvrc.sh: File not found\n";
   } else
-    printk("twin: RunTwEnvRC(): delaying .twenvrc.sh execution until secure mode ends.\n");
+    log(ERROR) << "twin: RunTwEnvRC(): delaying twenvrc.sh execution until secure mode ends.\n";
+  FreeMem(path);
 }
 
 /* remove const from a pointer and suppress compiler warnings */
