@@ -9,28 +9,26 @@
 #ifndef TWIN_WM_H
 #define TWIN_WM_H
 
-typedef struct wm_ctx wm_ctx;
-
 struct wm_ctx {
-  widget W; /* top level widget user is interacting with */
-  screen Screen;
-  menu Menu;
-  menuitem Item;
-  widget DW; /* deepest widget user is interacting with */
-  udat Type; /* Msg->Type: MSG_* */
-  byte Pos;  /* POS_* relative to (W) */
-  byte ByMouse;
+  Twidget W; /* top level Twidget user is interacting with */
+  Tscreen Screen;
+  Tmenu Menu;
+  Tmenuitem Item;
+  Twidget DW; /* deepest Twidget user is interacting with */
+  udat Type;  /* Tmsg->Type: MSG_* */
+  byte Pos;   /* POS_* relative to (W) */
+  bool ByMouse;
   udat Code; /* keyboard key / mouse buttons pressed */
   udat ShiftFlags;
   dat i, j;                /* mouse coordinates */
-  ldat Left, Up, Rgt, Dwn; /* window (W) corners */
+  ldat Left, Up, Rgt, Dwn; /* Twindow (W) corners */
 };
 
-byte InitWM(void);
+bool InitWM(void);
 
-void AskCloseWidget(widget W);
-void Check4Resize(window W);
-void MaximizeWindow(window W, byte full_screen);
+void AskCloseWidget(Twidget w);
+void Check4Resize(Twindow window);
+void MaximizeWindow(Twindow w, byte full_screen);
 void ShowWinList(wm_ctx *C);
 
 void FocusCtx(wm_ctx *C);
@@ -38,4 +36,6 @@ byte ActivateCtx(wm_ctx *C, byte State);
 
 void ForceRelease(const wm_ctx *C);
 
+tpos WMFindBorderWindow(Twindow w, dat u, dat v, byte border, tcell *ptr_cell);
+
 #endif /* TWIN_WM_H */
